! =================================================================================================================================
! MODULE       : stomate_io
!
! CONTACT      : orchidee-help _at_ listes.ipsl.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Module for read and write of restart files for all stomate modules.
!!
!!\n DESCRIPTION : This module contains the subroutines readstart and writerestart. All variables that will be read or written
!!                 are passed as argument to the subroutines. The subroutine readstart is called from stomate_initialize and 
!!                 writerestart is called from stomate_finalize.
!!                 Note: Not all variables saved in the start files are absolutely necessary. However, Sechiba's and Stomate's 
!!                 PFTs are not necessarily identical, and for that case this information needs to be saved.
!!
!!
!! RECENT CHANGE(S) : None
!!
!! REFERENCE(S)	: None
!!
!! SVN :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================
MODULE stomate_io
  USE stomate_data
  USE constantes
  USE constantes_soil
  USE mod_orchidee_para
  USE ioipsl_para 
  !-
  IMPLICIT NONE
  !-
  PRIVATE
  PUBLIC readstart, writerestart, get_reftemp_clear
  !-
  ! first call?
  !-
  LOGICAL,SAVE :: firstcall_io = .TRUE.
!$OMP THREADPRIVATE(firstcall_io)
  !-
  ! reference temperature (K)
  !-
  REAL(r_std),ALLOCATABLE,DIMENSION(:),SAVE :: trefe
!$OMP THREADPRIVATE(trefe)
  !-
CONTAINS

!! ================================================================================================================================
!! SUBROUTINE   : readstart
!!
!>\BRIEF        Read all variables for stomate from restart file. 
!!
!! DESCRIPTION  : Read all variables for stomate from restart file. 
!!                Initialize the variables if they were not found in the restart file or if there was no restart file.
!!                
!! \n
!_ ================================================================================================================================

  SUBROUTINE readstart &
       & (npts, index, lalo, resolution, t2m, dt_days, date, &
       &  ind, adapted, regenerate, moiavail_daily, gdd_init_date, litterhum_daily, &
       &  t2m_daily, t2m_min_daily, &
       !spitfire
       &  t2m_max_daily, wspeed_daily,&
       !endspit
       &  tsurf_daily, tsoil_daily, &
       &  soilhum_daily, precip_daily, gpp_daily, npp_daily, &
       &  turnover_daily, moiavail_month, moiavail_week,&
       &  t2m_longterm, tau_longterm, t2m_month, t2m_week,&
       &  tsoil_month, soilhum_month, fireindex, firelitter,  &
       &  maxmoiavail_lastyear, maxmoiavail_thisyear, &
       &  minmoiavail_lastyear, minmoiavail_thisyear, &
       &  maxgppweek_lastyear, maxgppweek_thisyear, &
       &  gdd0_lastyear, gdd0_thisyear, precip_lastyear, precip_thisyear, &
       &  gdd_m5_dormance,  gdd_from_growthinit, gdd_midwinter, ncd_dormance, ngd_minus5, &
       &  PFTpresent, npp_longterm, lm_lastyearmax, lm_thisyearmax, &
       &  maxfpc_lastyear, maxfpc_thisyear, &
       &  turnover_longterm, gpp_week, biomass, resp_maint_part, &
       &  leaf_age, leaf_frac, senescence, when_growthinit, age, &
       &  resp_hetero, tot_soil_resp, resp_maint, resp_growth, co2_fire, co2_to_bm_dgvm, &
       &  veget_lastlight, everywhere, need_adjacent, RIP_time, &
       &  time_hum_min, hum_min_dormance, &
       &  litterpart, litter_above, litter_below, dead_leaves, &
       &  carbon, &!SIMON commented here lignin_struc,&
!ORCHIDOC
       & carbon_32l, DOC, lignin_struc_above, lignin_struc_below, &
       !spitfire
       &  ni_acc,fuel_1hr,fuel_10hr,fuel_100hr,fuel_1000hr, & 
       !endspit
       & turnover_time, &
       &  prod10,prod100,flux10, flux100, &
       &  convflux, cflux_prod10, cflux_prod100, bm_to_litter, carb_mass_total,&
       &  Tseason, Tseason_length, Tseason_tmp, &
       &  Tmin_spring_time, begin_leaves, onset_date, &
       &  global_years, ok_equilibrium, nbp_accu, nbp_flux, &
       &  MatrixV, VectorU, previous_stock, current_stock, assim_param, &
       &  deepC_a, deepC_s, deepC_p, O2_soil, CH4_soil, O2_snow, CH4_snow, &
       &  thawed_humidity, depth_organic_soil, altmax, fixed_cryoturbation_depth, & !pss+
       &  uo_0, uold2_0, uo_wet1, uold2_wet1, uo_wet2, uold2_wet2, uo_wet3, uold2_wet3, &
       &  uo_wet4, uold2_wet4, tsurf_year, &!) !pss-
!gmjc
       &  wshtotsum, sr_ugb, sla_calc, nb_ani, grazed_frac, &
       &  import_yield, t2m_14, litter_not_avail, nb_grazingdays, &
       &  after_snow, after_wet, wet1day, wet2day, &
!end gmjc
!ORCHIDOC 
       &  interception_storage)
    !---------------------------------------------------------------------
    !- read start file
    !---------------------------------------------------------------------
    !-
    ! 0 declarations
    !-
    ! 0.1 input
    !-
    ! Domain size
    INTEGER(i_std),INTENT(in) :: npts
    ! Indices of the points on the map
    INTEGER(i_std),DIMENSION(npts),INTENT(in) :: index
    ! Geogr. coordinates (latitude,longitude) (degrees)
    REAL(r_std),DIMENSION(npts,2),INTENT(in) :: lalo
    ! size in x an y of the grid (m)
    REAL(r_std),DIMENSION(npts,2),INTENT(in) :: resolution
    REAL(r_std),DIMENSION(npts),INTENT(in)   :: t2m                !! 2 m air temperature from forcing file or coupled model (K)
    !-
    ! 0.2 output
    !-
    ! time step of STOMATE in days
    REAL(r_std),INTENT(out) :: dt_days
    ! date (d)
    INTEGER(i_std),INTENT(out) :: date
    ! density of individuals (1/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: ind
    ! Winter too cold? between 0 and 1
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: adapted
    ! Winter sufficiently cold? between 0 and 1
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: regenerate
    ! daily moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: moiavail_daily
    ! date for beginning of gdd count
    REAL(r_std),DIMENSION(npts,2),INTENT(out) :: gdd_init_date
    ! daily litter humidity
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: litterhum_daily
    ! daily 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: t2m_daily
    ! daily minimum 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: t2m_min_daily
    !spitfire
    ! daily maximum 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(out) :: t2m_max_daily
    ! daily wind speed(m/s)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: wspeed_daily
    !endspit
    ! daily surface temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: tsurf_daily
    ! daily soil temperatures (K)
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(out) :: tsoil_daily
    ! daily soil humidity
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(out) :: soilhum_daily
    ! daily precipitations (mm/day) (for phenology)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: precip_daily
    ! daily gross primary productivity (gC/m**2/day)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: gpp_daily
    ! daily net primary productivity (gC/m**2/day)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: npp_daily
    ! daily turnover rates (gC/m**2/day)
    REAL(r_std),DIMENSION(npts,nvm,nparts,nelements),INTENT(out) :: turnover_daily
    ! "monthly" moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: moiavail_month
    ! "weekly" moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: moiavail_week
    ! "long term" 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: t2m_longterm
    ! "tau_longterm"
    REAL(r_std), INTENT(out)        :: tau_longterm
    ! "monthly" 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: t2m_month
    ! "seasonal" 2 meter temperatures (K) 
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: Tseason
    ! temporary variable to calculate Tseason
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: Tseason_length
    ! temporary variable to calculate Tseason
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: Tseason_tmp

    REAL(r_std),DIMENSION(npts,nvm),INTENT(out)  :: Tmin_spring_time
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out)  :: onset_date
    LOGICAL,DIMENSION(npts,nvm),INTENT(out)      :: begin_leaves

    ! "weekly" 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: t2m_week
    ! "monthly" soil temperatures (K)
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(out) :: tsoil_month
    ! "monthly" soil humidity
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(out) :: soilhum_month
    ! Probability of fire
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: fireindex
    ! Longer term total litter above the ground, gC/m**2 of ground
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: firelitter
    ! last year's maximum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: maxmoiavail_lastyear
    ! this year's maximum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: maxmoiavail_thisyear
    ! last year's minimum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: minmoiavail_lastyear
    ! this year's minimum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: minmoiavail_thisyear
    ! last year's maximum weekly GPP
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: maxgppweek_lastyear
    ! this year's maximum weekly GPP
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: maxgppweek_thisyear
    ! last year's annual GDD0
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: gdd0_lastyear
    ! this year's annual GDD0
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: gdd0_thisyear
    ! last year's annual precipitation (mm/year)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: precip_lastyear
    ! this year's annual precipitation (mm/year)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: precip_thisyear
    ! growing degree days, threshold -5 deg C (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: gdd_m5_dormance
    ! growing degree days, from begin of season
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: gdd_from_growthinit
    ! growing degree days since midwinter (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: gdd_midwinter
    ! number of chilling days since leaves were lost (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: ncd_dormance
    ! number of growing days, threshold -5 deg C (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: ngd_minus5
    ! PFT exists (equivalent to fpc_max > 0 for natural PFTs)
    LOGICAL,DIMENSION(npts,nvm),INTENT(out)    :: PFTpresent
    ! "long term" net primary productivity (gC/m**2/year)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: npp_longterm
    ! last year's maximum leaf mass, for each PFT (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: lm_lastyearmax
    ! this year's maximum leaf mass, for each PFT (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: lm_thisyearmax
    ! last year's maximum fpc for each natural PFT, on ground
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: maxfpc_lastyear
    ! this year's maximum fpc for each PFT,
    ! on *total* ground (see stomate_season)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: maxfpc_thisyear
    ! "long term" turnover rate (gC/m**2/year)
    REAL(r_std),DIMENSION(npts,nvm,nparts,nelements),INTENT(out) :: turnover_longterm
    ! "weekly" GPP (gC/day/(m**2 covered)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: gpp_week
    ! biomass (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm,nparts,nelements),INTENT(out) :: biomass
    ! maintenance resp (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm,nparts),INTENT(out) :: resp_maint_part
    ! leaf age (days)
    REAL(r_std),DIMENSION(npts,nvm,nleafages),INTENT(out) :: leaf_age
    ! fraction of leaves in leaf age class
    REAL(r_std),DIMENSION(npts,nvm,nleafages),INTENT(out) :: leaf_frac
    ! is the plant senescent ? 
    !(only for deciduous trees - carbohydrate reserve)
    LOGICAL,DIMENSION(npts,nvm),INTENT(out) :: senescence
    ! how many days ago was the beginning of the growing season
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: when_growthinit
    ! mean age (years)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: age
    ! heterotrophic respiration (gC/day/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: resp_hetero
    ! maintenance respiration (gC/day/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: tot_soil_resp
    ! total soil respiration (gC/day/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: resp_maint
    ! growth respiration (gC/day/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: resp_growth
    ! carbon emitted into the atmosphere by fire (living and dead biomass)
    ! (in gC/m**2/time step)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: co2_fire
    ! biomass uptaken (gC/(m**2 of total ground)/day)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: co2_to_bm_dgvm
    ! vegetation fractions (on ground) after last light competition
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: veget_lastlight
    ! is the PFT everywhere in the grid box or very localized
    ! (after its introduction)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: everywhere
    ! in order for this PFT to be introduced,
    ! does it have to be present in an adjacent grid box?
    LOGICAL,DIMENSION(npts,nvm),INTENT(out) :: need_adjacent
    ! How much time ago was the PFT eliminated for the last time (y)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: RIP_time
    ! time elapsed since strongest moisture availability (d)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: time_hum_min
    ! minimum moisture during dormance
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: hum_min_dormance
    ! fraction of litter above the ground belonging to different PFTs
    ! separated for natural and agricultural PFTs.
    REAL(r_std),DIMENSION(npts,nvm,nlitt),INTENT(out) :: litterpart
    ! metabolic and structural litter, natural and agricultural,
    ! above and below ground (gC/m**2)
    REAL(r_std),DIMENSION(npts,nlitt,nvm,nelements),INTENT(out):: litter_above
    REAL(r_std),DIMENSION(npts,nlitt,nvm,ndeep,nelements),INTENT(out):: litter_below    
    ! dead leaves on ground, per PFT, metabolic and structural,
    ! in gC/(m**2 of ground)
    REAL(r_std),DIMENSION(npts,nvm,nlitt),INTENT(out) :: dead_leaves
    ! carbon pool: active, slow, or passive, (gC/m**2)
    REAL(r_std),DIMENSION(npts,ncarb,nvm),INTENT(out) :: carbon
     ! carbon pool 11-layer scheme: active, slow, or passive, (gC/m**2)  
    REAL(r_std),DIMENSION(npts,ncarb,nvm,ndeep),INTENT(out) :: carbon_32l
    !spitfire
    ! Nesterov index accumulated
    REAL(r_std), DIMENSION(npts), INTENT(out)                    :: ni_acc
    ! fuel classes (1, 10, 100, 1000 hours)
    REAL(r_std), DIMENSION(npts,nvm,nlitt,nelements), INTENT(out)               :: fuel_1hr
    REAL(r_std), DIMENSION(npts,nvm,nlitt,nelements), INTENT(out)               :: fuel_10hr
    REAL(r_std), DIMENSION(npts,nvm,nlitt,nelements), INTENT(out)               :: fuel_100hr
    REAL(r_std), DIMENSION(npts,nvm,nlitt,nelements), INTENT(out)               :: fuel_1000hr
    !endspit
    ! ratio Lignine/Carbon in structural litter, above and below ground,(gC/m**2)
!!SIMON ADDED ORCHIDOC
!    REAL(r_std),DIMENSION(npts,nvm,nlevs),INTENT(out) :: lignin_struc
    REAL(r_std),DIMENSION(npts,nvm,ndeep,ndoc,npool,nelements),INTENT(out) :: DOC
    ! ratio Lignine/Carbon in structural litter, above ground,(gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: lignin_struc_above
    ! ratio Lignine/Carbon in structural litter, below ground,(gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm,ndeep),INTENT(out) :: lignin_struc_below

    ! TF-DOC
    ! Storage of soluble OC and DOC attached to canopy (gC/m**2 of ground/dt)
    REAL(r_std),DIMENSION(npts,nvm,nelements),INTENT(out) :: interception_storage 
!!!ORCHIDOC END
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: turnover_time
!!!ORCHIDOC END
    ! For Spinup matrix resolution
    INTEGER(i_std), INTENT(out) :: global_years   
    LOGICAL, DIMENSION(npts), INTENT(out) :: ok_equilibrium
    REAL(r_std), DIMENSION(npts), INTENT(out) :: nbp_accu  !! Accumulated Net Biospheric Production over the year
    REAL(r_std), DIMENSION(npts), INTENT(out) :: nbp_flux  !! Net Biospheric Production over the year
    !-
    REAL(r_std), DIMENSION(npts,nvm,npool,npool), INTENT(out) :: MatrixV
    REAL(r_std), DIMENSION(npts,nvm,npool), INTENT(out) :: VectorU
    REAL(r_std), DIMENSION(npts,nvm,npool), INTENT(out) :: previous_stock
    REAL(r_std), DIMENSION(npts,nvm,npool), INTENT(out) :: current_stock    
    REAL(r_std), DIMENSION(npts,nvm,npco2),   INTENT(out) :: assim_param

    !-
    REAL(r_std), DIMENSION(npts,ndeep,nvm),INTENT(inout) :: deepC_a
    REAL(r_std), DIMENSION(npts,ndeep,nvm),INTENT(inout) :: deepC_s
    REAL(r_std), DIMENSION(npts,ndeep,nvm),INTENT(inout) :: deepC_p
    REAL(r_std), DIMENSION(npts,ndeep,nvm),INTENT(inout) :: O2_soil
    REAL(r_std), DIMENSION(npts,ndeep,nvm),INTENT(inout) :: CH4_soil
    REAL(r_std), DIMENSION(npts,nsnow,nvm),INTENT(inout) :: O2_snow
    REAL(r_std), DIMENSION(npts,nsnow,nvm),INTENT(inout) :: CH4_snow
    REAL(r_std), DIMENSION(npts),INTENT(inout)           :: thawed_humidity
    REAL(r_std), DIMENSION(npts),INTENT(inout)           :: depth_organic_soil
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)      :: altmax     !! Active layer thickness (m) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)      :: fixed_cryoturbation_depth !! Depth to hold cryoturbation to for fixed runs 

    !pss:+
    REAL(r_std),DIMENSION(npts,nvert),INTENT(out) :: uo_0
    REAL(r_std),DIMENSION(npts,nvert),INTENT(out)  :: uold2_0
    REAL(r_std),DIMENSION(npts,nvert),INTENT(out) :: uo_wet1
    REAL(r_std),DIMENSION(npts,nvert),INTENT(out)  :: uold2_wet1
    REAL(r_std),DIMENSION(npts,nvert),INTENT(out) :: uo_wet2
    REAL(r_std),DIMENSION(npts,nvert),INTENT(out)  :: uold2_wet2
    REAL(r_std),DIMENSION(npts,nvert),INTENT(out) :: uo_wet3
    REAL(r_std),DIMENSION(npts,nvert),INTENT(out)  :: uold2_wet3
    REAL(r_std),DIMENSION(npts,nvert),INTENT(out) :: uo_wet4
    REAL(r_std),DIMENSION(npts,nvert),INTENT(out)  :: uold2_wet4
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: tsurf_year
    !pss:-



    !-
    ! 0.3 not necessarily output
    !-
!gmjc
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out)    ::  sla_calc
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out)    ::  wshtotsum
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)  ::  sr_ugb
!    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)  ::  compt_ugb
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out)    ::  nb_ani
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)  ::  grazed_frac
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)  ::  import_yield
    REAL(r_std),DIMENSION(npts),INTENT(out)        ::  t2m_14
    REAL(r_std), DIMENSION(npts,nlitt,nvm), INTENT(out)  ::  litter_not_avail
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)  ::  nb_grazingdays
    REAL(r_std),DIMENSION(npts),INTENT(out)        :: after_snow
    REAL(r_std),DIMENSION(npts),INTENT(out)        :: after_wet
    REAL(r_std),DIMENSION(npts),INTENT(out)        :: wet1day
    REAL(r_std),DIMENSION(npts),INTENT(out)        :: wet2day
!end gmjc
    !-
    ! 0.4 local
    !-
    ! date, real
    REAL(r_std) :: date_real
    ! PFT exists (equivalent to fpc_max > 0 for natural PFTs), real
    REAL(r_std),DIMENSION(npts,nvm) :: PFTpresent_real
    ! is the plant senescent ?
    ! (only for deciduous trees - carbohydrate reserve), real
    REAL(r_std),DIMENSION(npts,nvm) :: senescence_real

    REAL(r_std),DIMENSION(npts,nvm) :: begin_leaves_real
    ! in order for this PFT to be introduced,
    ! does it have to be present in an adjacent grid box? - real
    REAL(r_std),DIMENSION(npts,nvm) :: need_adjacent_real
    REAL(r_std), DIMENSION(1) :: vartmp  !! temporary variable because restget/restput needs an array and not a scalar
    ! To store variables names for I/O
    CHARACTER(LEN=80) :: var_name
    ! string suffix indicating an index
    CHARACTER(LEN=10) :: part_str
    ! string suffix indicating biomass type for senecense
    CHARACTER(LEN=10) :: box_str
    ! string suffix indicating litter type
    CHARACTER(LEN=3),DIMENSION(nlitt) :: litter_str
    ! string suffix indicating level
    CHARACTER(LEN=2),DIMENSION(nlevs) :: level_str
    CHARACTER(LEN=2) :: layer_str
    ! temporary storage
    REAL(r_std),DIMENSION(1) :: xtmp
    ! index
    INTEGER(i_std) :: j,k,l,m,h
    ! reference temperature (K)

    CHARACTER(LEN=1),DIMENSION(nelements) :: element_str   !! string suffix indicating element
    REAL(r_std), DIMENSION(1) :: temp_global_years
    CHARACTER(LEN=6), DIMENSION(npool) :: pools_str
    REAL(r_std), DIMENSION(npts) :: ok_equilibrium_real    
    CHARACTER(LEN=80) :: filename   !! filename for reference temerature file
    ! Permafrost carbon processes
    LOGICAL :: read_input_deepC_a
    LOGICAL :: read_input_deepC_s
    LOGICAL :: read_input_deepC_p
    LOGICAL :: read_input_thawed_humidity
    LOGICAL :: read_input_depth_organic_soil
    real(r_std) :: deepC_a_init, deepC_s_init, deepC_p_init
    real(r_std) :: thawed_humidity_input = 0.5
    LOGICAL ::  reset_thawed_humidity = .FALSE.

    ! Wetland CH4
    INTEGER(i_std) :: nivo !pss:+-
    REAL(r_std) :: CH4atmo_CONC !pss:+-


    ! land cover change variables 
    ! products remaining in the 10/100 year-turnover pool after the annual release for each compartment
    ! (10 or 100 + 1 : input from year of land cover change)
    REAL(r_std),DIMENSION(npts,0:10,nwp),INTENT(out)                           :: prod10
    REAL(r_std),DIMENSION(npts,0:100,nwp),INTENT(out)                          :: prod100
    ! annual release from the 10/100 year-turnover pool compartments
    REAL(r_std),DIMENSION(npts,10,nwp),INTENT(out)                           :: flux10
    REAL(r_std),DIMENSION(npts,100,nwp),INTENT(out)                          :: flux100
    REAL(r_std), DIMENSION(npts,nwp), INTENT(out)                            :: convflux
    REAL(r_std), DIMENSION(npts,nwp), INTENT(out)                            :: cflux_prod10
    REAL(r_std), DIMENSION(npts,nwp), INTENT(out)                            :: cflux_prod100
    REAL(r_std),DIMENSION(npts,nvm,nparts,nelements),INTENT(out)         :: bm_to_litter
    REAL(r_std),DIMENSION(npts),INTENT(out)                              :: carb_mass_total
    REAL(r_std),DIMENSION(npts,nvm)                                      :: vcmax_tmp

    !---------------------------------------------------------------------
    IF (printlev >= 3) WRITE(numout,*) 'Entering readstart'

    ! near surface CH4 concentration
    !pss:+
    CH4atmo_CONC=0.0017
    CALL GETIN('CH4atmo_CONC', CH4atmo_CONC)
    !pss:-

    !-
    ! 1 string definitions
    !-
    DO l=1,nlitt
       IF     (l == imetabolic) THEN
          litter_str(l) = 'met'
       ELSEIF (l == istructural) THEN
          litter_str(l) = 'str'
       ELSE
          CALL ipslerr_p(3,'stomate_io readstart', 'Define litter_str','','')
       ENDIF
    ENDDO
    !-
    DO l=1,nlevs
       IF     (l == iabove) THEN
          level_str(l) = 'ab'
       ELSEIF (l == ibelow) THEN
          level_str(l) = 'be'
       ELSE
          CALL ipslerr_p(3,'stomate_io readstart','Define level_str','','')
       ENDIF
    ENDDO

    pools_str(1:npool) =(/'str_ab','str_be','met_ab','met_be','actif ','slow  ','passif'/)

    !-
    DO l=1,nelements
       IF     (l == icarbon) THEN
          element_str(l) = ''
!!$       ELSEIF (l == initrogen) THEN
!!$          element_str(l) = '_n'
       ELSE
          CALL ipslerr_p(3,'stomate_io readstart','Define element_str','','')
       ENDIF
    ENDDO
    !-
    ! 2 run control
    !-
    ! 2.2 time step of STOMATE in days
    !-
    CALL restget_p(rest_id_stomate, 'dt_days', itime, .TRUE., un, dt_days)
    !-
    ! 2.3 date
    !-
    CALL restget_p (rest_id_stomate, 'date', itime, .TRUE., zero, date_real)
    date = NINT(date_real)
    !-
    ! 3 daily meteorological variables
    !-
    moiavail_daily(:,:) = val_exp
    var_name = 'moiavail_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., moiavail_daily, 'gather', nbp_glo, index_g)
    IF (ALL(moiavail_daily(:,:) == val_exp)) moiavail_daily(:,:) = zero
    !-
    gdd_init_date(:,:) = val_exp
    var_name = 'gdd_init_date'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 2 , 1, itime, &
         &              .TRUE., gdd_init_date, 'gather', nbp_glo, index_g)
    IF (ALL(gdd_init_date(:,1) == val_exp)) gdd_init_date(:,1) = 365.
    !-
    litterhum_daily(:) = val_exp
    var_name = 'litterhum_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., litterhum_daily, 'gather', nbp_glo, index_g)
    IF (ALL(litterhum_daily(:) == val_exp)) litterhum_daily(:) = zero
    !-
    t2m_daily(:) = val_exp
    var_name = 't2m_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., t2m_daily, 'gather', nbp_glo, index_g)
    IF (ALL(t2m_daily(:) == val_exp)) t2m_daily(:) = zero
    !-
    t2m_min_daily(:) = val_exp
    var_name = 't2m_min_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., t2m_min_daily, 'gather', nbp_glo, index_g)
    IF (ALL(t2m_min_daily(:) == val_exp)) t2m_min_daily(:) = large_value
    !spitfire
    !-
    t2m_max_daily(:) = val_exp
    var_name = 't2m_max_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., t2m_max_daily, 'gather', nbp_glo, index_g)
    IF (ALL(t2m_max_daily(:) == val_exp)) t2m_max_daily(:) = (-1.) * large_value
    !-
    wspeed_daily(:) = val_exp
    var_name = 'wspeed_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., wspeed_daily, 'gather', nbp_glo, index_g)
    IF (ALL(wspeed_daily(:) == val_exp)) wspeed_daily(:) = large_value
    !endspit
    !-
    tsurf_daily(:) = val_exp
    var_name = 'tsurf_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., tsurf_daily, 'gather', nbp_glo, index_g)
    ! The initial value is set to the current temperature at 2m
    IF (ALL(tsurf_daily(:) == val_exp)) tsurf_daily(:) = t2m(:)
    !-
    tsoil_daily(:,:) = val_exp
    var_name = 'tsoil_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nbdl, 1, itime, &
         &                .TRUE., tsoil_daily, 'gather', nbp_glo, index_g)
    IF (ALL(tsoil_daily(:,:) == val_exp)) tsoil_daily(:,:) = zero
    !-
    soilhum_daily(:,:) = val_exp
    var_name = 'soilhum_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nbdl, 1, itime, &
         &                .TRUE., soilhum_daily, 'gather', nbp_glo, index_g)
    IF (ALL(soilhum_daily(:,:) == val_exp)) soilhum_daily(:,:) = zero
    !-
    precip_daily(:) = val_exp
    var_name = 'precip_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., precip_daily, 'gather', nbp_glo, index_g)
    IF (ALL(precip_daily(:) == val_exp)) precip_daily(:) = zero
    !-
    ! 4 productivities
    !-
    gpp_daily(:,:) = val_exp
    var_name = 'gpp_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., gpp_daily, 'gather', nbp_glo, index_g)
    IF (ALL(gpp_daily(:,:) == val_exp)) gpp_daily(:,:) = zero
    !-
    npp_daily(:,:) = val_exp
    var_name = 'npp_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., npp_daily, 'gather', nbp_glo, index_g)
    IF (ALL(npp_daily(:,:) == val_exp)) npp_daily(:,:) = zero
    !-
    turnover_daily(:,:,:,:) = val_exp
    CALL restget_p (rest_id_stomate, 'turnover_daily', nbp_glo, nvm, nparts, nelements, itime, &
          &                .TRUE., turnover_daily, 'gather', nbp_glo, index_g)
    IF (ALL(turnover_daily == val_exp)) turnover_daily = zero
    !-
    ! 5 monthly meteorological variables
    !-
    moiavail_month(:,:) = val_exp
    var_name = 'moiavail_month'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., moiavail_month, 'gather', nbp_glo, index_g)
    IF (ALL(moiavail_month(:,:) == val_exp)) moiavail_month(:,:) = zero
    !-
    moiavail_week(:,:) = val_exp
    var_name = 'moiavail_week'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., moiavail_week, 'gather', nbp_glo, index_g)
    IF (ALL(moiavail_week(:,:) == val_exp)) moiavail_week(:,:) = zero
    

    !
    ! Longterm temperature at 2m
    !
    var_name = 't2m_longterm'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., t2m_longterm, 'gather', nbp_glo, index_g)

    IF (ALL(t2m_longterm(:) == val_exp)) THEN
       ! t2m_longterm is not in restart file
       ! It is possible to initialize it from a file REFTEMP_FILE or initialize it to t2m(default). 

       !Config Key   = REFTEMP_FILE
       !Config Desc  = Name of file from which the reference temperature can be read
       !Config If    = OK_STOMATE
       !Config Def   = NONE
       !Config Help  = Specify filename if the initial reference temperature should be 
       !Config         read from file. Otherwise, the reference temperature is calculated. 
       !Config Units = [FILE]
       !---
       filename = 'NONE'
       CALL getin_p('REFTEMP_FILE',filename)
       IF ( filename == 'NONE' ) THEN
          ! The initial value for the reference temperature is set to the current temperature
          t2m_longterm(:)=t2m(:)
          ! Set the counter to 2 time steps
          tau_longterm=2
       ELSE
          ! t2m_longterm will be initialized with the values from REFTEMP_FILE
          CALL get_reftemp(npts, lalo, resolution, filename, t2m_longterm)
          tau_longterm=tau_longterm_max
       END IF
    ELSE
       ! t2m_longterm was in the restart file
       ! Now read tau_longterm
       ! tau_longterm is a scalar, therefor only master process read this value
       CALL restget_p (rest_id_stomate, 'tau_longterm', itime, .TRUE., val_exp, tau_longterm)
       IF (tau_longterm == val_exp) THEN
             ! tau_longterm is not found in restart file. 
             ! This is not normal as t2m_longterm was in restart file. Write a warning and initialize it to tau_longterm_max
          CALL ipslerr(2, 'stomate_io readstart','tau_longterm was not in restart file',&
               'But t2m_longterm was in restart file','')
          tau_longterm = tau_longterm_max
       END IF

    END IF
    !-
    t2m_month(:) = val_exp
    var_name = 't2m_month'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., t2m_month, 'gather', nbp_glo, index_g)
    IF (ALL(t2m_month(:) == val_exp)) t2m_month(:) = t2m(:)
    
    Tseason(:) = val_exp
    var_name = 'Tseason'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., Tseason, 'gather', nbp_glo, index_g)
    IF (ALL(Tseason(:) == val_exp)) Tseason(:) = t2m(:)
    !-
    Tseason_length(:) = val_exp
    var_name = 'Tseason_length'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., Tseason_length, 'gather', nbp_glo, index_g)
    IF (ALL(Tseason_length(:) == val_exp)) Tseason_length(:) = t2m(:)
    !-
    Tseason_tmp(:) = val_exp
    var_name = 'Tseason_tmp'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., Tseason_tmp, 'gather', nbp_glo, index_g)
    IF (ALL(Tseason_tmp(:) == val_exp)) Tseason_tmp(:) = t2m(:)
    !-

    Tmin_spring_time(:,:) = val_exp
    var_name = 'Tmin_spring_time'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              .TRUE., Tmin_spring_time, 'gather', nbp_glo, index_g)
    IF (ALL(Tmin_spring_time(:,:) == val_exp)) Tmin_spring_time(:,:) = zero
    !-

    onset_date(:,:) = val_exp
    var_name = 'onset_date'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., onset_date, 'gather', nbp_glo, index_g)
    IF (ALL(onset_date == val_exp)) onset_date(:,:) = zero
    !-

    t2m_week(:) = val_exp
    var_name = 't2m_week'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., t2m_week, 'gather', nbp_glo, index_g)
    ! The initial value is set to the current temperature
    IF (ALL(t2m_week(:) == val_exp)) t2m_week(:) = t2m(:)
    
    tsoil_month(:,:) = val_exp
    var_name = 'tsoil_month'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nbdl, 1, itime, &
         &              .TRUE., tsoil_month, 'gather', nbp_glo, index_g)

    ! The initial value is set to the current temperature
    IF (ALL(tsoil_month(:,:) == val_exp)) THEN
       DO l=1,nbdl
          tsoil_month(:,l) = t2m(:)
       ENDDO
    ENDIF
    !-
    soilhum_month(:,:) = val_exp
    var_name = 'soilhum_month'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nbdl, 1, itime, &
         &              .TRUE., soilhum_month, 'gather', nbp_glo, index_g)
    IF (ALL(soilhum_month(:,:) == val_exp)) soilhum_month(:,:) = zero
    !-
    ! 6 fire probability
    !-
    fireindex(:,:) = val_exp
    var_name = 'fireindex'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              .TRUE., fireindex, 'gather', nbp_glo, index_g)
    IF (ALL(fireindex(:,:) == val_exp)) fireindex(:,:) = zero
    !-
    firelitter(:,:) = val_exp
    var_name = 'firelitter'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              .TRUE., firelitter, 'gather', nbp_glo, index_g)
    IF (ALL(firelitter(:,:) == val_exp)) firelitter(:,:) = zero
    !-
    ! 7 maximum and minimum moisture availabilities for tropic phenology
    !-
    maxmoiavail_lastyear(:,:) = val_exp
    var_name = 'maxmoistr_last'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxmoiavail_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxmoiavail_lastyear(:,:) == val_exp)) &
         &     maxmoiavail_lastyear(:,:) = zero
    !-
    maxmoiavail_thisyear(:,:) = val_exp
    var_name = 'maxmoistr_this'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxmoiavail_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxmoiavail_thisyear(:,:) == val_exp)) &
         &     maxmoiavail_thisyear(:,:) = zero
    !-
    minmoiavail_lastyear(:,:) = val_exp
    var_name = 'minmoistr_last'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., minmoiavail_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(minmoiavail_lastyear(:,:) == val_exp)) &
         &     minmoiavail_lastyear(:,:) = un
    !-
    minmoiavail_thisyear(:,:) = val_exp
    var_name = 'minmoistr_this'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., minmoiavail_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL( minmoiavail_thisyear(:,:) == val_exp)) &
         &     minmoiavail_thisyear(:,:) = un
    !-
    ! 8 maximum "weekly" GPP
    !-
    maxgppweek_lastyear(:,:) = val_exp
    var_name = 'maxgppweek_lastyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxgppweek_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxgppweek_lastyear(:,:) == val_exp)) &
         &     maxgppweek_lastyear(:,:) = zero
    !-
    maxgppweek_thisyear(:,:) = val_exp
    var_name = 'maxgppweek_thisyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxgppweek_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxgppweek_thisyear(:,:) == val_exp)) &
         &     maxgppweek_thisyear(:,:) = zero
    !-
    ! 9 annual GDD0
    !-
    gdd0_thisyear(:) = val_exp
    var_name = 'gdd0_thisyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., gdd0_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL(gdd0_thisyear(:) == val_exp)) gdd0_thisyear(:) = zero
    !-
    gdd0_lastyear(:) = val_exp
    var_name = 'gdd0_lastyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., gdd0_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(gdd0_lastyear(:) == val_exp)) gdd0_lastyear(:) = gdd_crit_estab
    !-
    ! 10 annual precipitation
    !-
    precip_thisyear(:) = val_exp
    var_name = 'precip_thisyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., precip_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL(precip_thisyear(:) == val_exp)) precip_thisyear(:) = zero
    !-
    precip_lastyear(:) = val_exp
    var_name = 'precip_lastyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., precip_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(precip_lastyear(:) == val_exp)) &
         &     precip_lastyear(:) = precip_crit
    !-
    ! 11 derived "biometeorological" variables
    !-
    gdd_m5_dormance(:,:) = val_exp
    var_name = 'gdd_m5_dormance'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., gdd_m5_dormance, 'gather', nbp_glo, index_g)
    IF (ALL(gdd_m5_dormance(:,:) == val_exp)) &
         &     gdd_m5_dormance(:,:) = undef
    !-
    gdd_from_growthinit(:,:) = val_exp
    var_name = 'gdd_from_growthinit'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., gdd_from_growthinit, 'gather', nbp_glo, index_g)
    IF (ALL(gdd_from_growthinit(:,:) == val_exp)) &
         &     gdd_from_growthinit(:,:) = zero
    !-
    gdd_midwinter(:,:) = val_exp
    var_name = 'gdd_midwinter'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., gdd_midwinter, 'gather', nbp_glo, index_g)
    IF (ALL(gdd_midwinter(:,:) == val_exp)) gdd_midwinter(:,:) = undef
    !-
    ncd_dormance(:,:) = val_exp
    var_name = 'ncd_dormance'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., ncd_dormance, 'gather', nbp_glo, index_g)
    IF (ALL(ncd_dormance(:,:) == val_exp)) ncd_dormance(:,:) = undef
    !-
    ngd_minus5(:,:) = val_exp
    var_name = 'ngd_minus5'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., ngd_minus5, 'gather', nbp_glo, index_g)
    IF (ALL(ngd_minus5(:,:) == val_exp)) ngd_minus5(:,:) = zero
    !-
    time_hum_min(:,:) = val_exp
    var_name = 'time_hum_min'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., time_hum_min, 'gather', nbp_glo, index_g)
    IF (ALL(time_hum_min(:,:) == val_exp)) time_hum_min(:,:) = undef
    !-
    hum_min_dormance(:,:) = val_exp
    var_name = 'hum_min_dormance'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., hum_min_dormance, 'gather', nbp_glo, index_g)
    IF (ALL(hum_min_dormance(:,:) == val_exp)) &
         &     hum_min_dormance(:,:) = undef
    !-
    ! 12 Plant status
    !-
    PFTpresent_real(:,:) = val_exp
    var_name = 'PFTpresent'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., PFTpresent_real, 'gather', nbp_glo, index_g)
    IF (ALL(PFTpresent_real(:,:) == val_exp)) PFTpresent_real(:,:) = zero
    WHERE (PFTpresent_real(:,:) >= .5)
       PFTpresent = .TRUE.
    ELSEWHERE
       PFTpresent = .FALSE.
    ENDWHERE
    !-
    ind(:,:) = val_exp
    var_name = 'ind'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., ind, 'gather', nbp_glo, index_g)
    IF (ALL(ind(:,:) == val_exp)) ind(:,:) = zero
    !-
    adapted(:,:) = val_exp
    var_name = 'adapted'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., adapted, 'gather', nbp_glo, index_g)
    IF (ALL(adapted(:,:) == val_exp)) adapted(:,:) = zero
    !-
    regenerate(:,:) = val_exp
    var_name = 'regenerate'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., regenerate, 'gather', nbp_glo, index_g)
    IF (ALL(regenerate(:,:) == val_exp)) regenerate(:,:) = zero
    !-
    npp_longterm(:,:) = val_exp
    var_name = 'npp_longterm'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., npp_longterm, 'gather', nbp_glo, index_g)
    IF (ALL(npp_longterm(:,:) == val_exp)) npp_longterm(:,:) = zero
    !-
    lm_lastyearmax(:,:) = val_exp
    var_name = 'lm_lastyearmax'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., lm_lastyearmax, 'gather', nbp_glo, index_g)
    IF (ALL(lm_lastyearmax(:,:) == val_exp)) lm_lastyearmax(:,:) = zero
    !-
    lm_thisyearmax(:,:) = val_exp
    var_name = 'lm_thisyearmax'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., lm_thisyearmax, 'gather', nbp_glo, index_g)
    IF (ALL(lm_thisyearmax(:,:) == val_exp)) lm_thisyearmax(:,:) = zero
    !-
    maxfpc_lastyear(:,:) = val_exp
    var_name = 'maxfpc_lastyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxfpc_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxfpc_lastyear(:,:) == val_exp)) maxfpc_lastyear(:,:) = zero
    !-
    maxfpc_thisyear(:,:) = val_exp
    var_name = 'maxfpc_thisyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxfpc_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxfpc_thisyear(:,:) == val_exp)) maxfpc_thisyear(:,:) = zero
    !-
    turnover_time(:,:) = val_exp
    var_name = 'turnover_time'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., turnover_time, 'gather', nbp_glo, index_g)
    IF ( ALL( turnover_time(:,:) == val_exp)) turnover_time(:,:) = 100.
    !-
    turnover_longterm(:,:,:,:) = val_exp
    CALL restget_p (rest_id_stomate, 'turnover_longterm', nbp_glo, nvm, nparts, nelements, itime, &
         &              .TRUE., turnover_longterm, 'gather', nbp_glo, index_g)
    IF (ALL(turnover_longterm == val_exp)) turnover_longterm = zero
    !-
    gpp_week(:,:) = val_exp
    var_name = 'gpp_week'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., gpp_week, 'gather', nbp_glo, index_g)
    IF (ALL(gpp_week(:,:) == val_exp)) gpp_week(:,:) = zero
    !-
    biomass(:,:,:,:) = val_exp
    CALL restget_p (rest_id_stomate, 'biomass', nbp_glo, nvm, nparts, nelements, itime,  &
         &                   .TRUE., biomass, 'gather', nbp_glo, index_g)
    IF (ALL(biomass == val_exp)) biomass = zero
    !-
    resp_maint_part(:,:,:) = val_exp
    var_name = 'maint_resp'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , nparts, itime, &
         &                   .TRUE., resp_maint_part, 'gather', nbp_glo, index_g)
    IF (ALL(resp_maint_part == val_exp)) resp_maint_part(:,:,:) = zero
    !-
    leaf_age(:,:,:) = val_exp
    var_name = 'leaf_age'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , nleafages, itime, &
         &                   .TRUE., leaf_age, 'gather', nbp_glo, index_g)
    IF (ALL(leaf_age == val_exp)) leaf_age(:,:,:) = zero
    !-
    leaf_frac(:,:,:) = val_exp
    var_name = 'leaf_frac'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , nleafages, itime, &
         &                  .TRUE., leaf_frac, 'gather', nbp_glo, index_g)
    IF (ALL(leaf_frac == val_exp)) leaf_frac(:,:,:) = zero
    !-
    senescence_real(:,:) = val_exp
    var_name = 'senescence'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., senescence_real, 'gather', nbp_glo, index_g)
    IF (ALL(senescence_real(:,:) == val_exp)) senescence_real(:,:) = zero
    WHERE ( senescence_real(:,:) >= .5 )
       senescence = .TRUE.
    ELSEWHERE
       senescence = .FALSE.
    ENDWHERE

    begin_leaves_real(:,:) = val_exp
    var_name = 'begin_leaves'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., begin_leaves_real, 'gather', nbp_glo, index_g)
    IF (ALL(begin_leaves_real(:,:) == val_exp)) begin_leaves_real(:,:) = zero
    WHERE ( begin_leaves_real(:,:) >= .5 )
       begin_leaves = .TRUE.
    ELSEWHERE
       begin_leaves = .FALSE.
    ENDWHERE
    !-
    when_growthinit(:,:) = val_exp
    var_name = 'when_growthinit'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., when_growthinit, 'gather', nbp_glo, index_g)
    IF (ALL(when_growthinit(:,:) == val_exp)) &
         &     when_growthinit(:,:) = zero
    !-
    age(:,:) = val_exp
    var_name = 'age'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., age, 'gather', nbp_glo, index_g)
    IF (ALL(age(:,:) == val_exp)) age(:,:) = zero
    !-
    ! 13 CO2
    !-
    resp_hetero(:,:) = val_exp
    var_name = 'resp_hetero'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                .TRUE., resp_hetero, 'gather', nbp_glo, index_g)
    IF (ALL(resp_hetero(:,:) == val_exp)) resp_hetero(:,:) = zero
    !-
    tot_soil_resp(:,:) = val_exp
    var_name = 'tot_soil_resp'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                .TRUE., tot_soil_resp, 'gather', nbp_glo, index_g)
    IF (ALL(tot_soil_resp(:,:) == val_exp)) tot_soil_resp(:,:) = zero
    !-
    resp_maint(:,:) = val_exp
    var_name = 'resp_maint'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., resp_maint, 'gather', nbp_glo, index_g)
    IF (ALL(resp_maint(:,:) == val_exp)) resp_maint(:,:) = zero
    !-
    resp_growth(:,:) = val_exp
    var_name = 'resp_growth'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., resp_growth, 'gather', nbp_glo, index_g)
    IF (ALL(resp_growth(:,:) == val_exp)) resp_growth(:,:) = zero
    !-
    co2_fire(:,:) = val_exp
    var_name = 'co2_fire'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm     , 1, itime, &
         &                .TRUE., co2_fire, 'gather', nbp_glo, index_g)
    IF (ALL(co2_fire(:,:) == val_exp)) co2_fire(:,:) = zero
    !-
    co2_to_bm_dgvm(:,:) = val_exp
    var_name = 'co2_to_bm_dgvm'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm     , 1, itime, &
         &                .TRUE., co2_to_bm_dgvm, 'gather', nbp_glo, index_g)
    IF (ALL(co2_to_bm_dgvm(:,:) == val_exp)) co2_to_bm_dgvm(:,:) = zero
    !-
    ! 14 vegetation distribution after last light competition
    !-
    veget_lastlight(:,:) = val_exp
    var_name = 'veget_lastlight'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., veget_lastlight, 'gather', nbp_glo, index_g)
    IF (ALL(veget_lastlight(:,:) == val_exp)) veget_lastlight(:,:) = zero
    !-
    ! 15 establishment criteria
    !-
    everywhere(:,:) = val_exp
    var_name = 'everywhere'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., everywhere, 'gather', nbp_glo, index_g)
    IF (ALL(everywhere(:,:) == val_exp)) everywhere(:,:) = zero
    !-
    need_adjacent_real(:,:) = val_exp
    var_name = 'need_adjacent'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., need_adjacent_real, 'gather', nbp_glo, index_g)
    IF (ALL(need_adjacent_real(:,:) == val_exp)) &
         &     need_adjacent_real(:,:) = zero
    WHERE ( need_adjacent_real(:,:) >= .5 )
       need_adjacent = .TRUE.
    ELSEWHERE
       need_adjacent = .FALSE.
    ENDWHERE
    !-
    RIP_time(:,:) = val_exp
    var_name = 'RIP_time'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., RIP_time, 'gather', nbp_glo, index_g)
    IF (ALL(RIP_time(:,:) == val_exp)) RIP_time(:,:) = large_value
    !-
    ! 17 litter
    !-
    litterpart(:,:,:) = val_exp
    var_name = 'litterpart'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , nlitt, itime, &
         &                   .TRUE., litterpart, 'gather', nbp_glo, index_g)
    IF (ALL(litterpart == val_exp)) litterpart(:,:,:) = zero
    !-
    litter_above(:,:,:,:) = val_exp
    CALL restget_p (rest_id_stomate, 'litter_above', nbp_glo, nlitt , nvm, nelements, itime, &
         &                     .TRUE., litter_above, 'gather', nbp_glo, index_g)
    IF (ALL(litter_above == val_exp)) litter_above = zero
    !-
    litter_below(:,:,:,:,:) = val_exp
    CALL restget_p (rest_id_stomate, 'litter_below', nbp_glo, nlitt, nvm, ndeep, nelements, itime, &
         &                     .TRUE., litter_below, 'gather', nbp_glo, index_g)
    IF (ALL(litter_below == val_exp)) litter_below = zero
    !-
    dead_leaves(:,:,:) = val_exp
    var_name = 'dead_leaves'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , nlitt, itime, &
         &                   .TRUE., dead_leaves, 'gather', nbp_glo, index_g)
    IF (ALL(dead_leaves == val_exp)) dead_leaves(:,:,:) = zero
    !-
    carbon(:,:,:) = val_exp
    var_name = 'carbon'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, ncarb , nvm, itime, &
         &                   .TRUE., carbon, 'gather', nbp_glo, index_g)
    IF (ALL(carbon == val_exp)) carbon(:,:,:) = zero
    !-
!     carbon_32l(:,:,:,:) = val_exp
!     var_name = 'carbon_32l'
!     CALL restget_p (rest_id_stomate, var_name, nbp_glo, ncarb , nvm, itime, &
!          &                   .TRUE., carbon_32l, 'gather', nbp_glo, index_g)
!     IF (ALL(carbon_32l == val_exp)) carbon_32l(:,:,:,:) = zero
!!!SIMON ADDED ORCHIDOC



!    
!    var_name = 'carbon_32l'
!    CALL restget_p (rest_id_stomate, var_name, nbp_glo, ncarb , nvm, ndeep, itime, &
!        &                   .TRUE., carbon_32l, 'gather', nbp_glo, index_g)


    carbon_32l = val_exp
    var_name = 'carbon_32l_a'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, ndeep, itime, &
         &               .TRUE., carbon_32l(:,iactive,:,:), 'gather', nbp_glo, index_g)

    var_name = 'carbon_32l_s'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo,  nvm, ndeep, itime, &
         &               .TRUE., carbon_32l(:,islow,:,:), 'gather', nbp_glo, index_g)
    var_name = 'carbon_32l_p'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo,  nvm, ndeep, itime, &
         &               .TRUE., carbon_32l(:,ipassive,:,:), 'gather', nbp_glo, index_g) 
    IF (ALL(carbon_32l == val_exp)) carbon_32l = zero

    !
    !-
    DOC(:,:,:,:,:,:) = val_exp
    var_name = 'freedoc'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, ndeep, npool, nelements, itime, &
        &                   .TRUE., DOC(:,:,:,ifree,:,:), 'gather', nbp_glo, index_g)
    IF (ALL(DOC(:,:,:,ifree,:,:)== val_exp))DOC(:,:,:,ifree,:,:) = zero
    !-
    var_name = 'adsdoc'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm ,ndeep, npool, nelements, itime, &
         &                   .TRUE., DOC(:,:,:,iadsorbed,:,:), 'gather', nbp_glo, index_g)
    IF (ALL(DOC(:,:,:,iadsorbed,:,:)== val_exp))DOC(:,:,:,iadsorbed,:,:) = zero
    !-
    lignin_struc_above(:,:) = val_exp
    var_name = 'lignin_struc_above'
    CALL restget_p &
        &    (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
        &     .TRUE., lignin_struc_above(:,:), 'gather', nbp_glo, index_g) 
    IF (ALL(lignin_struc_above(:,:) == val_exp)) lignin_struc_above(:,:) = zero 
    !- 
    lignin_struc_below(:,:,:) = val_exp
    CALL restget_p &
        &    (rest_id_stomate, 'lig_struc_be', nbp_glo, nvm, ndeep, itime, &
        &     .TRUE., lignin_struc_below, 'gather', nbp_glo, index_g)
    IF (ALL(lignin_struc_below == val_exp)) lignin_struc_below = zero

   ! TF-DOC
   interception_storage(:,:,:) = val_exp
   var_name = 'interception_storage'
   CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nvm, 1, itime, &
        &         &                .TRUE., interception_storage(:,:,icarbon), 'gather', nbp_glo, index_g)
   IF (ALL(interception_storage(:,:,icarbon) == val_exp)) interception_storage(:,:,icarbon) = zero  


    !

!     lignin_struc(:,:,:) = val_exp
!     var_name = 'lignin_struc'
!     CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, nlevs, itime, &
!          &     .TRUE., lignin_struc, 'gather', nbp_glo, index_g)
!     IF (ALL(lignin_struc == val_exp)) lignin_struc(:,:,:) = zero
! !!SIMON END ORCHIDOC
     !spitfire
      ni_acc(:) = val_exp
        var_name = 'ni_acc'
        CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
     &              .TRUE., ni_acc, 'gather', nbp_glo, index_g)
        IF (ALL(ni_acc(:) == val_exp)) ni_acc(:) = zero
    
      fuel_1hr(:,:,:,:) = val_exp
      CALL restget_p (rest_id_stomate, 'fuel_1hr', nbp_glo, nvm, nlitt, nelements, itime, &
             &       .TRUE., fuel_1hr, 'gather', nbp_glo, index_g)
      IF (ALL(fuel_1hr == val_exp)) fuel_1hr = 0.0

      fuel_10hr(:,:,:,:) = val_exp
      CALL restget_p (rest_id_stomate, 'fuel_10hr', nbp_glo, nvm, nlitt, nelements, itime, &
             &       .TRUE., fuel_10hr, 'gather', nbp_glo, index_g)
      IF (ALL(fuel_10hr == val_exp)) fuel_10hr = 0.0

      fuel_100hr(:,:,:,:) = val_exp
      CALL restget_p (rest_id_stomate, 'fuel_100hr', nbp_glo, nvm  , nlitt, nelements, itime, &
            &       .TRUE., fuel_100hr, 'gather', nbp_glo, index_g)
      IF (ALL(fuel_100hr == val_exp)) fuel_100hr = 0.0

     CALL restget_p (rest_id_stomate, 'fuel_1000hr', nbp_glo, nvm  , nlitt, nelements, itime, &
             &       .TRUE., fuel_1000hr, 'gather', nbp_glo, index_g)
     IF (ALL(fuel_1000hr == val_exp)) fuel_1000hr = 0.0
     !endspit
    
    !-
    ! 18 land cover change
    !-
    prod10(:,:,:) = val_exp
    var_name = 'prod10'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 11     , nwp, itime, &
         &                .TRUE., prod10, 'gather', nbp_glo, index_g)
    IF (ALL(prod10(:,:,:) == val_exp)) prod10(:,:,:) = zero

    prod100(:,:,:) = val_exp
    var_name = 'prod100'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 101     , nwp, itime, &
         &                .TRUE., prod100, 'gather', nbp_glo, index_g)
    IF (ALL(prod100(:,:,:) == val_exp)) prod100(:,:,:) = zero


    flux10(:,:,:) = val_exp
    var_name = 'flux10'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 10     , nwp, itime, &
         &                .TRUE., flux10, 'gather', nbp_glo, index_g)
    IF (ALL(flux10(:,:,:) == val_exp)) flux10(:,:,:) = zero

    flux100(:,:,:) = val_exp
    var_name = 'flux100'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 100     , nwp, itime, &
         &                .TRUE., flux100, 'gather', nbp_glo, index_g)
    IF (ALL(flux100(:,:,:) == val_exp)) flux100(:,:,:) = zero

!!!SIMON ADDED ORCHIDOC
	convflux(:,:) = zero
	cflux_prod10(:,:) = zero
	cflux_prod100(:,:) = zero

  !  convflux(:,:) = val_exp
  !  var_name = 'convflux'
  !  CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1, nwp, itime, &
  !       &              .TRUE., convflux, 'gather', nbp_glo, index_g)
  !  IF (ALL(convflux(:,:) == val_exp)) convflux(:,:) = zero

  !  cflux_prod10(:,:) = val_exp
  !  var_name = 'cflux_prod10'
  !  CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , nwp, itime, &
  !       &              .TRUE., cflux_prod10, 'gather', nbp_glo, index_g)
  !  IF (ALL(cflux_prod10(:,:) == val_exp)) cflux_prod10(:,:) = zero

!    cflux_prod100(:,:) = val_exp
!    var_name = 'cflux_prod100'
!    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , nwp, itime, &
!         &              .TRUE., cflux_prod100, 'gather', nbp_glo, index_g)
!    IF (ALL(cflux_prod100(:,:) == val_exp)) cflux_prod100(:,:) = zero
!
    bm_to_litter(:,:,:,:) = val_exp
    CALL restget_p (rest_id_stomate, 'bm_to_litter', nbp_glo, nvm  , nparts, nelements, itime, &
        &                .TRUE., bm_to_litter, 'gather', nbp_glo, index_g)
    IF (ALL(bm_to_litter == val_exp)) bm_to_litter = zero

    carb_mass_total(:) = val_exp
    var_name = 'carb_mass_total'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., carb_mass_total, 'gather', nbp_glo, index_g)
    IF (ALL(carb_mass_total(:) == val_exp)) carb_mass_total(:) = zero

    !Permafrost carbon related
    deepC_a(:,:,:) = val_exp
    var_name = 'deepC_a'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, ndeep, nvm, itime, &
         &               .TRUE., deepC_a, 'gather', nbp_glo, index_g)
    IF (ALL(deepC_a == val_exp)) deepC_a(:,:,:) = zero !deepC_a_init

    deepC_s(:,:,:) = val_exp
    var_name = 'deepC_s'
    CALL restget_p (rest_id_stomate,var_name, nbp_glo, ndeep, nvm, itime, &
         &               .TRUE., deepC_s, 'gather', nbp_glo, index_g)
    IF (ALL(deepC_s == val_exp)) deepC_s(:,:,:) = zero !deepC_s_init

    deepC_p(:,:,:) = val_exp
    var_name = 'deepC_p'
    CALL restget_p (rest_id_stomate,var_name, nbp_glo, ndeep, nvm, itime, &
         &               .TRUE., deepC_p, 'gather', nbp_glo, index_g)
    IF (ALL(deepC_p == val_exp)) deepC_p(:,:,:) = zero !deepC_p_init

    O2_soil(:,:,:) = val_exp
    var_name = 'O2_soil'
    CALL restget_p (rest_id_stomate,var_name, nbp_glo, ndeep, nvm, itime, &
           &               .TRUE., O2_soil, 'gather', nbp_glo, index_g)
    IF (ALL(O2_soil == val_exp)) O2_soil(:,:,:) = O2_init_conc

    CH4_soil(:,:,:) = val_exp
    var_name = 'CH4_soil'
    CALL restget_p (rest_id_stomate,var_name, nbp_glo, ndeep, nvm, itime, &
         &               .TRUE., CH4_soil, 'gather', nbp_glo, index_g)
    IF (ALL(CH4_soil == val_exp)) CH4_soil(:,:,:) = CH4_init_conc

    O2_snow(:,:,:) = val_exp
    var_name = 'O2_snow'
    CALL restget_p (rest_id_stomate,var_name, nbp_glo, nsnow, nvm, itime, &
         &               .TRUE., O2_snow, 'gather', nbp_glo, index_g)
    IF (ALL(O2_snow == val_exp)) O2_snow(:,:,:) = O2_init_conc

    CH4_snow(:,:,:) = val_exp
    var_name = 'CH4_snow'
    CALL restget_p (rest_id_stomate,var_name, nbp_glo, nsnow, nvm, itime, &
         &               .TRUE., CH4_snow, 'gather', nbp_glo, index_g)
    IF (ALL(CH4_snow == val_exp)) CH4_snow(:,:,:) = CH4_init_conc

  thawed_humidity(:) = val_exp
  var_name = 'thawed_humidity'

  CALL getin('reset_thawed_humidity', reset_thawed_humidity)
  if ( reset_thawed_humidity ) then
     CALL getin('thawed_humidity_input', thawed_humidity_input)
     thawed_humidity(:) = thawed_humidity_input
  else
     CALL restget_p (rest_id_stomate,var_name, nbp_glo, 1, 1, itime, &
          &               .TRUE., thawed_humidity, 'gather', nbp_glo, index_g)
     IF (ALL(thawed_humidity(:) == val_exp)) THEN
        thawed_humidity(:) = thawed_humidity_input
        read_input_thawed_humidity = .TRUE.
     ENDIF
  endif


  depth_organic_soil(:) = val_exp
  var_name = 'depth_organic_soil'
  CALL restget_p (rest_id_stomate,var_name, nbp_glo, 1, 1, itime, &
       &               .TRUE., depth_organic_soil, 'gather', nbp_glo, index_g)
  IF (ALL(depth_organic_soil(:) == val_exp)) THEN
     depth_organic_soil(:) = 0.0
     read_input_depth_organic_soil = .TRUE.
  ENDIF

  altmax(:,:) = val_exp
  var_name = 'altmax'
  CALL restget_p (rest_id_stomate,var_name, nbp_glo, nvm, 1, itime, &
       &               .TRUE., altmax, 'gather', nbp_glo, index_g)
  IF (ALL(altmax(:,:) == val_exp)) THEN
     altmax(:,:) = 0.0
  ENDIF

  fixed_cryoturbation_depth(:,:) = val_exp
  var_name = 'fixed_cryoturb_depth'
  CALL restget_p (rest_id_stomate,var_name, nbp_glo, nvm, 1, itime, &
       &               .TRUE., fixed_cryoturbation_depth, 'gather', nbp_glo, index_g)
  IF (ALL(fixed_cryoturbation_depth(:,:) == val_exp)) THEN
     fixed_cryoturbation_depth(:,:) = 0.0
  ENDIF

  !-

!!!!! Wetland CH4 methane
    !pss:+
    uo_0(:,:) = val_exp
    var_name = 'uo_0'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nvert, 1, itime, &
         &              .TRUE.,uo_0 , 'gather', nbp_glo, index_g)
    IF (ALL(uo_0(:,:) == val_exp)) THEN
       DO nivo=1,nvert
          IF (nivo .LE. ns) THEN
            uo_0(:,nivo) = scmax
         ELSE
            uo_0(:,nivo) = CH4atmo_CONC
         ENDIF
      ENDDO
   ENDIF

   uold2_0(:,:) = val_exp
   var_name = 'uold2_0'
   CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nvert, 1, itime, &
        &              .TRUE.,uold2_0 , 'gather', nbp_glo, index_g)
   IF (ALL(uold2_0(:,:) == val_exp)) THEN
      DO nivo=1,nvert
         IF (nivo .LE. ns) THEN
            uold2_0(:,nivo) = scmax
         ELSE
            uold2_0(:,nivo) = CH4atmo_CONC
         ENDIF
      ENDDO
   ENDIF
   
   uo_wet1(:,:) = val_exp
   var_name = 'uo_wet1'
   CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nvert, 1, itime, &
       &              .TRUE.,uo_wet1 , 'gather', nbp_glo, index_g)
   IF (ALL(uo_wet1(:,:) == val_exp)) THEN
      DO nivo=1,nvert
         IF (nivo .LE. ns-10) THEN
            uo_wet1(:,nivo) = scmax
         ELSE
            uo_wet1(:,nivo) = CH4atmo_CONC
         ENDIF
      ENDDO
   ENDIF
   
   uold2_wet1(:,:) = val_exp
   var_name = 'uold2_wet1'
   CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nvert, 1, itime, &
        &              .TRUE.,uold2_wet1 , 'gather', nbp_glo, index_g)
   IF (ALL(uold2_wet1(:,:) == val_exp)) THEN
      DO nivo=1,nvert
         IF (nivo .LE. ns-10) THEN
            uold2_wet1(:,nivo) = scmax
         ELSE
            uold2_wet1(:,nivo) = CH4atmo_CONC
         ENDIF
      ENDDO
   ENDIF
  
   uo_wet2(:,:) = val_exp
   var_name = 'uo_wet2'
   CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nvert, 1, itime, &
        &              .TRUE.,uo_wet2 , 'gather', nbp_glo, index_g)
   IF (ALL(uo_wet2(:,:) == val_exp)) THEN
      DO nivo=1,nvert
         IF (nivo .LE. ns-10) THEN
            uo_wet2(:,nivo) = scmax
         ELSE
            uo_wet2(:,nivo) = CH4atmo_CONC
         ENDIF
      ENDDO
   ENDIF

   uold2_wet2(:,:) = val_exp
   var_name = 'uold2_wet2'
   CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nvert, 1, itime, &
        &              .TRUE.,uold2_wet2 , 'gather', nbp_glo, index_g)
   IF (ALL(uold2_wet2(:,:) == val_exp)) THEN
      DO nivo=1,nvert
         IF (nivo .LE. ns-10) THEN
            uold2_wet2(:,nivo) = scmax
         ELSE
            uold2_wet2(:,nivo) = CH4atmo_CONC
         ENDIF
      ENDDO
   ENDIF

   uo_wet3(:,:) = val_exp
   var_name = 'uo_wet3'
   CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nvert, 1, itime, &
        &              .TRUE.,uo_wet3 , 'gather', nbp_glo, index_g)
   IF (ALL(uo_wet3(:,:) == val_exp)) THEN
      DO nivo=1,nvert
         IF (nivo .LE. ns-10) THEN
            uo_wet3(:,nivo) = scmax
         ELSE
            uo_wet3(:,nivo) = CH4atmo_CONC
         ENDIF
      ENDDO
   ENDIF
   
   uold2_wet3(:,:) = val_exp
   var_name = 'uold2_wet3'
   CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nvert, 1, itime, &
        &              .TRUE.,uold2_wet3 , 'gather', nbp_glo, index_g)
   IF (ALL(uold2_wet3(:,:) == val_exp)) THEN
      DO nivo=1,nvert
         IF (nivo .LE. ns-10) THEN
            uold2_wet3(:,nivo) = scmax
         ELSE
            uold2_wet3(:,nivo) = CH4atmo_CONC
         ENDIF
      ENDDO
   ENDIF

   uo_wet4(:,:) = val_exp
   var_name = 'uo_wet4'
   CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nvert, 1, itime, &
        &              .TRUE.,uo_wet4 , 'gather', nbp_glo, index_g)
   IF (ALL(uo_wet4(:,:) == val_exp)) THEN
      DO nivo=1,nvert
         IF (nivo .LE. ns-10) THEN
            uo_wet4(:,nivo) = scmax
         ELSE
            uo_wet4(:,nivo) = CH4atmo_CONC
         ENDIF
      ENDDO
   ENDIF

   uold2_wet4(:,:) = val_exp
   var_name = 'uold2_wet4'
   CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nvert, 1, itime, &
        &              .TRUE.,uold2_wet4 , 'gather', nbp_glo, index_g)
   IF (ALL(uold2_wet4(:,:) == val_exp)) THEN
      DO nivo=1,nvert
         IF (nivo .LE. ns-10) THEN
            uold2_wet4(:,nivo) = scmax
         ELSE
            uold2_wet4(:,nivo) = CH4atmo_CONC
         ENDIF
      ENDDO
   ENDIF
   
   tsurf_year(:) = val_exp
   var_name = 'tsurf_year'
   CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
        &              .TRUE., tsurf_year, 'gather', nbp_glo, index_g)
   IF (ALL(tsurf_year(:) == val_exp)) tsurf_year(:) = t2m(:)
   
!pss:-
!gmjc
!-
!  sla_calc(:,:) = val_exp
!  var_name = 'sla_calc'
!  CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm   , 1, itime, &
! &              .TRUE., sla_calc, 'gather', nbp_glo, index_g)
!  IF (ALL(sla_calc(:,:) == val_exp)) THEN
!     DO j=2,nvm
!        sla_calc(:,j) = sla(j)
!     END DO
!  END IF
WRITE(*,*) 'before read gmjc'
  wshtotsum(:,:) = val_exp
  var_name = 'wshtotsum'
  CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm     , 1, itime, &
 &              .TRUE., wshtotsum, 'gather', nbp_glo, index_g)
    IF (ALL(wshtotsum(:,:) == val_exp)) wshtotsum(:,:) = zero
!-
  sr_ugb(:,:) = val_exp
  var_name = 'sr_ugb'
  CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm     , 1, itime, &
 &              .TRUE., sr_ugb, 'gather', nbp_glo, index_g)
    IF (ALL(sr_ugb(:,:) == val_exp)) sr_ugb(:,:) = zero
!-
  sla_calc(:,:) = val_exp
  var_name = 'sla_calc'
  CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm     , 1, itime, &
 &              .TRUE., sla_calc, 'gather', nbp_glo, index_g)
!    IF (ALL(sla_calc(:,:) == val_exp)) sla_calc(:,:) = zero
  IF (ALL(sla_calc(:,:) == val_exp)) THEN
     DO j=1,nvm
        sla_calc(:,j) = sla(j)
     END DO
  END IF
!-
  nb_ani(:,:) = val_exp
  var_name = 'nb_ani'
  CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm     , 1, itime, &
 &              .TRUE., nb_ani, 'gather', nbp_glo, index_g)
    IF (ALL(nb_ani(:,:) == val_exp)) nb_ani(:,:) = zero
!-
  grazed_frac(:,:) = val_exp
  var_name = 'grazed_frac'
  CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm     , 1, itime, &
 &              .TRUE., grazed_frac, 'gather', nbp_glo, index_g)
    IF (ALL(grazed_frac(:,:) == val_exp)) grazed_frac(:,:) = zero
!-
  import_yield(:,:) = val_exp
  var_name = 'import_yield'
  CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm     , 1, itime, &
 &              .TRUE., import_yield, 'gather', nbp_glo, index_g)
    IF (ALL(import_yield(:,:) == val_exp)) import_yield(:,:) = zero

!-
   t2m_14(:) = val_exp
  var_name = 't2m_14'
  CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
 &              .TRUE., t2m_14, 'gather', nbp_glo, index_g)
  IF (ALL(t2m_14(:) == val_exp)) t2m_14(:) = t2m(:)
!

    litter_not_avail(:,:,:) = val_exp
    CALL restget_p (rest_id_stomate, 'litter_not_avail', nbp_glo, nlitt, nvm, itime, &
        &                   .TRUE., litter_not_avail, 'gather', nbp_glo, index_g)
    IF (ALL(litter_not_avail == val_exp)) litter_not_avail = zero
!-
  nb_grazingdays(:,:) = val_exp
  var_name = 'nb_grazingdays'
  CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm     , 1, itime, &
 &              .TRUE., nb_grazingdays, 'gather', nbp_glo, index_g)
    IF (ALL(nb_grazingdays(:,:) == val_exp)) nb_grazingdays(:,:) = zero

!-
  after_snow(:) = val_exp
  var_name = 'after_snow'
  CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
 &              .TRUE., after_snow, 'gather', nbp_glo, index_g)
    IF (ALL(after_snow(:) == val_exp)) after_snow(:) = zero
!-
  after_wet(:) = val_exp
  var_name = 'after_wet'
  CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
 &              .TRUE., after_wet, 'gather', nbp_glo, index_g)
    IF (ALL(after_wet(:) == val_exp)) after_wet(:) = zero
!-
  wet1day(:) = val_exp
  var_name = 'wet1day'
  CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
 &              .TRUE., wet1day, 'gather', nbp_glo, index_g)
    IF (ALL(wet1day(:) == val_exp)) wet1day(:) = 6
!-
  wet2day(:) = val_exp
  var_name = 'wet2day'
  CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
 &              .TRUE., wet2day, 'gather', nbp_glo, index_g)
    IF (ALL(wet2day(:) == val_exp)) wet2day(:) = 6
WRITE(*,*) 'after read gmjc'
!-
!end gmjc
    !-
    ! 19. Spinup
    !-
!    IF (spinup_analytic) THEN
       CALL restget_p (rest_id_stomate, 'Global_years', itime, .TRUE., zero, global_years)
       !global_years = INT(temp_global_years(1))

       nbp_accu(:) = val_exp
       var_name = 'nbp_sum'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
            &              .TRUE., nbp_accu, 'gather', nbp_glo, index_g)
       IF (ALL(nbp_accu(:) == val_exp)) nbp_accu(:) = zero    

       nbp_flux(:) = val_exp
       var_name = 'nbp_flux'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
            &              .TRUE., nbp_flux, 'gather', nbp_glo, index_g)
       IF (ALL(nbp_flux(:) == val_exp)) nbp_flux(:) = zero     

       !-
       ok_equilibrium_real(:) = val_exp
       var_name = 'ok_equilibrium'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo , 1  , 1, itime, &
            &                .TRUE., ok_equilibrium_real,'gather', nbp_glo, index_g)
       IF (ALL(ok_equilibrium_real(:) == val_exp)) ok_equilibrium_real(:) = zero
       WHERE(ok_equilibrium_real(:) >= 0.5) 
          ok_equilibrium = .TRUE.
       ELSEWHERE
          ok_equilibrium = .FALSE.
       ENDWHERE

       MatrixV(:,:,:,:) = val_exp
       CALL restget_p (rest_id_stomate, 'MatrixV', nbp_glo, nvm , npool, npool, itime, &
           &                     .TRUE., MatrixV, 'gather', nbp_glo, index_g)
       ! If nothing is found in the restart file, we initialize each submatrix by identity
       IF (ALL(MatrixV(:,:,:,:) == val_exp))  THEN 
          MatrixV(:,:,:,:) = zero
          DO l = 1,npool
             MatrixV(:,:,l,l) = un
          END DO
       END IF

       VectorU(:,:,:)  = val_exp
       var_name = 'Vector_U'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, npool, itime, &
         &     .TRUE., VectorU, 'gather', nbp_glo, index_g)
       IF (ALL(VectorU == val_exp))  VectorU(:,:,:) = zero
       
       previous_stock(:,:,:)  = val_exp
       var_name = 'previous_stock'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, npool, itime, &
            &     .TRUE., previous_stock, 'gather', nbp_glo, index_g)
       IF (ALL(previous_stock == val_exp))  previous_stock = undef_sechiba
       
       current_stock(:,:,:)  = val_exp
       var_name = 'current_stock'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, npool, itime, &
            &     .TRUE., current_stock, 'gather', nbp_glo, index_g)
       IF (ALL(current_stock == val_exp))  current_stock(:,:,:) = zero
 
         
!    ENDIF ! spinup_matrix_method

    ! Read assim_param from restart file. The initialization of assim_param will 
    ! be done in stomate_var_init if the variable is not in the restart file.
    assim_param(:,:,:)  = val_exp
    var_name = 'assim_param'
    CALL restget_p &
         &    (rest_id_stomate, var_name, nbp_glo, nvm, npco2, itime, &
         &     .TRUE., assim_param, 'gather', nbp_glo, index_g)
 
    IF (printlev >= 4) WRITE(numout,*) 'Leaving readstart'
    !-----------------------
  END SUBROUTINE readstart

!! ================================================================================================================================
!! SUBROUTINE   : writerestart
!!
!>\BRIEF        Write all variables for stomate from restart file. 
!!
!! DESCRIPTION  : Write all variables for stomate from restart file. 
!!                
!! \n
!_ ================================================================================================================================

  SUBROUTINE writerestart &
       & (npts, index, dt_days, date, &
       &  ind, adapted, regenerate, moiavail_daily, gdd_init_date, litterhum_daily, &
       &  t2m_daily, t2m_min_daily, &
       !spitfire
       &  t2m_max_daily, wspeed_daily, &
       !endspit
       &  tsurf_daily, tsoil_daily, &
       &  soilhum_daily, precip_daily, gpp_daily, npp_daily, &
       &  turnover_daily, moiavail_month, moiavail_week, &
       &  t2m_longterm, tau_longterm, t2m_month, t2m_week, &
       &  tsoil_month, soilhum_month, fireindex, firelitter, &
       &  maxmoiavail_lastyear, maxmoiavail_thisyear, &
       &  minmoiavail_lastyear, minmoiavail_thisyear, &
       &  maxgppweek_lastyear, maxgppweek_thisyear, &
       &  gdd0_lastyear, gdd0_thisyear, precip_lastyear, precip_thisyear, &
       &  gdd_m5_dormance, gdd_from_growthinit, gdd_midwinter, ncd_dormance, ngd_minus5, &
       &  PFTpresent, npp_longterm, lm_lastyearmax, lm_thisyearmax, &
       &  maxfpc_lastyear, maxfpc_thisyear, &
       &  turnover_longterm, gpp_week, biomass, resp_maint_part, &
       &  leaf_age, leaf_frac, senescence, when_growthinit, age, &
       &  resp_hetero, tot_soil_resp, resp_maint, resp_growth, co2_fire, co2_to_bm_dgvm, &
       &  veget_lastlight, everywhere, need_adjacent, RIP_time, &
       &  time_hum_min, hum_min_dormance, &
       &  litterpart, litter_above, litter_below, dead_leaves, &
       &  carbon, &
       !SIMON ORCHIDOC
       carbon_32l, DOC, lignin_struc_above, lignin_struc_below, & 
       !spitfire
       &  ni_acc,fuel_1hr,fuel_10hr,fuel_100hr,fuel_1000hr, & 
       !endspit
       & turnover_time, &
       &  prod10,prod100 ,flux10, flux100, &
       &  convflux, cflux_prod10, cflux_prod100, bm_to_litter, carb_mass_total, &
       &  Tseason, Tseason_length, Tseason_tmp, &
       &  Tmin_spring_time, begin_leaves, onset_date, &
       &  global_years, ok_equilibrium, nbp_accu, nbp_flux, &
       &  MatrixV, VectorU, previous_stock, current_stock, assim_param, &
       &  deepC_a, deepC_s, deepC_p, O2_soil, CH4_soil, O2_snow, CH4_snow, &
       &  thawed_humidity, depth_organic_soil, altmax, fixed_cryoturbation_depth, & !pss+
       &  uo_0, uold2_0, uo_wet1, uold2_wet1, uo_wet2, uold2_wet2, uo_wet3, uold2_wet3, &
       &  uo_wet4, uold2_wet4, tsurf_year, &!) !pss:-
!gmjc
       &  wshtotsum, sr_ugb, sla_calc, nb_ani, grazed_frac, &
       &  import_yield, t2m_14, litter_not_avail, nb_grazingdays, &
       &  after_snow, after_wet, wet1day, wet2day, &
!end gmjc
!SIMON: TF
       & interception_storage)
!ENDJCADD
    !---------------------------------------------------------------------
    !- write restart file
    !---------------------------------------------------------------------
    !-
    ! 0 declarations
    !-
    ! 0.1 input
    !-
    ! Domain size
    INTEGER(i_std),INTENT(in) :: npts
    ! Indices of the points on the map
    INTEGER(i_std),DIMENSION(npts),INTENT(in) :: index
    ! time step of STOMATE in days
    REAL(r_std),INTENT(in) :: dt_days
    ! date (d)
    INTEGER(i_std),INTENT(in) :: date
    ! density of individuals (1/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: ind
    ! Winter too cold? between 0 and 1
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: adapted
    ! Winter sufficiently cold? between 0 and 1
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: regenerate
    ! daily moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: moiavail_daily
    ! gdd init date
    REAL(r_std),DIMENSION(npts,2),INTENT(in) :: gdd_init_date
    ! daily litter humidity
    REAL(r_std),DIMENSION(npts),INTENT(in) :: litterhum_daily
    ! daily 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: t2m_daily
    ! daily minimum 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: t2m_min_daily
    !spitfire
    ! daily maximum 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(out) :: t2m_max_daily
    ! daily wind speed(m/s)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: wspeed_daily
    !endspit
    ! daily surface temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: tsurf_daily
    ! daily soil temperatures (K)
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(in) :: tsoil_daily
    ! daily soil humidity
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(in) :: soilhum_daily
    ! daily precipitations (mm/day) (for phenology)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: precip_daily
    ! daily gross primary productivity (gC/m**2/day)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: gpp_daily
    ! daily net primary productivity (gC/m**2/day)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: npp_daily
    ! daily turnover rates (gC/m**2/day)
    REAL(r_std),DIMENSION(npts,nvm,nparts,nelements),INTENT(in) :: turnover_daily
    ! "monthly" moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: moiavail_month
    ! "weekly" moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: moiavail_week
    ! "long term" 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: t2m_longterm
    ! "tau_longterm"
    REAL(r_std), INTENT(IN)             :: tau_longterm
    ! "monthly" 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: t2m_month
    ! "seasonal" 2 meter temperatures (K) 
    REAL(r_std),DIMENSION(npts),INTENT(in)      :: Tseason
    ! temporary variable to calculate Tseason
    REAL(r_std),DIMENSION(npts),INTENT(in)      :: Tseason_length
    ! temporary variable to calculate Tseason
    REAL(r_std),DIMENSION(npts),INTENT(in)      :: Tseason_tmp

    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)  :: Tmin_spring_time
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)  :: onset_date
    LOGICAL,DIMENSION(npts,nvm),INTENT(in)      :: begin_leaves

    ! "weekly" 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: t2m_week
    ! "monthly" soil temperatures (K)
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(in) :: tsoil_month
    ! "monthly" soil humidity
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(in) :: soilhum_month
    ! Probability of fire
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: fireindex
    ! Longer term total litter above the ground, gC/m**2 of ground
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: firelitter
    ! last year's maximum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: maxmoiavail_lastyear
    ! this year's maximum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: maxmoiavail_thisyear
    ! last year's minimum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: minmoiavail_lastyear
    ! this year's minimum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: minmoiavail_thisyear
    ! last year's maximum weekly GPP
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: maxgppweek_lastyear
    ! this year's maximum weekly GPP
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: maxgppweek_thisyear
    ! last year's annual GDD0
    REAL(r_std),DIMENSION(npts),INTENT(in) :: gdd0_lastyear
    ! this year's annual GDD0
    REAL(r_std),DIMENSION(npts),INTENT(in) :: gdd0_thisyear
    ! last year's annual precipitation (mm/year)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: precip_lastyear
    ! this year's annual precipitation (mm/year)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: precip_thisyear
    ! growing degree days, threshold -5 deg C (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: gdd_m5_dormance
    ! growing degree days, from begin of season (crops)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: gdd_from_growthinit
    ! growing degree days since midwinter (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: gdd_midwinter
    ! number of chilling days since leaves were lost (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: ncd_dormance
    ! number of growing days, threshold -5 deg C (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: ngd_minus5
    ! PFT exists (equivalent to fpc_max > 0 for natural PFTs)
    LOGICAL,DIMENSION(npts,nvm),INTENT(in) :: PFTpresent
    ! "long term" net primary productivity (gC/m**2/year)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: npp_longterm
    ! last year's maximum leaf mass, for each PFT (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: lm_lastyearmax
    ! this year's maximum leaf mass, for each PFT (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: lm_thisyearmax
    ! last year's maximum fpc for each natural PFT, on ground
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: maxfpc_lastyear
    ! this year's maximum fpc for each PFT,
    ! on *total* ground (see stomate_season)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: maxfpc_thisyear
    ! "long term" turnover rate (gC/m**2/year)
    REAL(r_std),DIMENSION(npts,nvm,nparts,nelements),INTENT(in) :: turnover_longterm
    ! "weekly" GPP (gC/day/(m**2 covered)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: gpp_week
    ! biomass (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm,nparts,nelements),INTENT(in) :: biomass
    ! maintenance respiration (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm,nparts),INTENT(in) :: resp_maint_part
    ! leaf age (days)
    REAL(r_std),DIMENSION(npts,nvm,nleafages),INTENT(in) :: leaf_age
    ! fraction of leaves in leaf age class
    REAL(r_std),DIMENSION(npts,nvm,nleafages),INTENT(in) :: leaf_frac
    ! is the plant senescent ?
    ! (only for deciduous trees - carbohydrate reserve)
    LOGICAL,DIMENSION(npts,nvm),INTENT(in) :: senescence
    ! how many days ago was the beginning of the growing season
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: when_growthinit
    ! mean age (years)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: age
    ! heterotrophic respiration (gC/day/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: resp_hetero
    ! maintenance respiration (gC/day/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: tot_soil_resp
    ! total soil respiration (gC/day/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: resp_maint
    ! growth respiration (gC/day/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: resp_growth
    ! carbon emitted into the atmosphere by fire (living and dead biomass)
    ! (in gC/m**2/time step)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: co2_fire
    ! biomass uptaken (gC/(m**2 of total ground)/day)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: co2_to_bm_dgvm
    ! vegetation fractions (on ground) after last light competition
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: veget_lastlight
    ! is the PFT everywhere in the grid box or very localized
    ! (after its introduction)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: everywhere
    ! in order for this PFT to be introduced,
    ! does it have to be present in an adjacent grid box?
    LOGICAL,DIMENSION(npts,nvm),INTENT(in) :: need_adjacent
    ! How much time ago was the PFT eliminated for the last time (y)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: RIP_time
    ! time elapsed since strongest moisture availability (d)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: time_hum_min
    ! minimum moisture during dormance
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: hum_min_dormance
    ! fraction of litter above the ground belonging to different PFTs
    REAL(r_std),DIMENSION(npts,nvm,nlitt),INTENT(in) :: litterpart
    ! metabolic and structural litter, above and below ground (gC/m**2)
    REAL(r_std),DIMENSION(npts,nlitt,nvm,nelements),INTENT(in) :: litter_above
    REAL(r_std),DIMENSION(npts,nlitt,nvm,ndeep,nelements),INTENT(in) :: litter_below    
    ! dead leaves on ground, per PFT, metabolic and structural,
    ! in gC/(m**2 of ground)
    REAL(r_std),DIMENSION(npts,nvm,nlitt),INTENT(in) :: dead_leaves
    ! carbon pool: active, slow, or passive, (gC/m**2)
    REAL(r_std),DIMENSION(npts,ncarb,nvm),INTENT(in) :: carbon
    ! carbon pool 11 layer scheme: active, slow, or passive, (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm,ndeep,ndoc,npool,nelements),INTENT(in) :: DOC
    REAL(r_std),DIMENSION(npts,ncarb,nvm,ndeep),INTENT(in) :: carbon_32l
   ! ! ratio Lignine/Carbon in structural litter, above and below ground, (gC/m**2)
   ! REAL(r_std),DIMENSION(npts,nvm,nlevs),INTENT(in) :: lignin_struc
      REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: lignin_struc_above
    ! ratio Lignine/Carbon in structural litter, below ground,(gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm,ndeep),INTENT(in) :: lignin_struc_below
    ! TF-DOC
    ! Storage of soluble OC and DOC attached to canopy (gC/m**2 of ground/dt)
    REAL(r_std),DIMENSION(npts,nvm,nelements),INTENT(in) :: interception_storage 

    ! turnover_time of leaves
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: turnover_time

    ! For Spinup matrix resolution
    INTEGER(i_std), INTENT(in) :: global_years   
    LOGICAL, DIMENSION(npts), INTENT(in) :: ok_equilibrium
    REAL(r_std), DIMENSION(npts), INTENT(in) :: nbp_accu  !! Accumulated Net Biospheric Production over the year 
    REAL(r_std), DIMENSION(npts), INTENT(in) :: nbp_flux  !! Net Biospheric Production over the year 
    !-
    REAL(r_std), DIMENSION(npts,nvm,npool,npool), INTENT(in) :: MatrixV
    REAL(r_std), DIMENSION(npts,nvm,npool), INTENT(in) :: VectorU
    REAL(r_std), DIMENSION(npts,nvm,npool), INTENT(in) :: previous_stock
    REAL(r_std), DIMENSION(npts,nvm,npool), INTENT(in) :: current_stock 
    ! Permafrost carbon related
    real(r_std), DIMENSION(npts,ndeep,nvm),INTENT(inout) :: deepC_a
    real(r_std), DIMENSION(npts,ndeep,nvm),intent(inout) :: deepC_s
    real(r_std), DIMENSION(npts,ndeep,nvm),intent(inout) :: deepC_p
    real(r_std), DIMENSION(npts,ndeep,nvm),intent(inout) :: O2_soil
    real(r_std), DIMENSION(npts,ndeep,nvm),intent(inout) :: CH4_soil
    real(r_std), DIMENSION(npts,nsnow,nvm),intent(inout) :: O2_snow
    real(r_std), DIMENSION(npts,nsnow,nvm),intent(inout) :: CH4_snow
    real(r_std), DIMENSION(npts),intent(inout)           :: thawed_humidity
    real(r_std), DIMENSION(npts),intent(inout)           :: depth_organic_soil
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)      :: altmax
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)      :: fixed_cryoturbation_depth
    REAL(r_std), DIMENSION(npts,nvm,npco2),   INTENT(in) :: assim_param
    ! Wetland CH4 methane
    !pss:+
    REAL(r_std),DIMENSION(npts,nvert),INTENT(in) :: uo_0
    REAL(r_std),DIMENSION(npts,nvert),INTENT(in)  :: uold2_0
    REAL(r_std),DIMENSION(npts,nvert),INTENT(in) :: uo_wet1
    REAL(r_std),DIMENSION(npts,nvert),INTENT(in)  :: uold2_wet1
    REAL(r_std),DIMENSION(npts,nvert),INTENT(in) :: uo_wet2
    REAL(r_std),DIMENSION(npts,nvert),INTENT(in)  :: uold2_wet2
    REAL(r_std),DIMENSION(npts,nvert),INTENT(in) :: uo_wet3
    REAL(r_std),DIMENSION(npts,nvert),INTENT(in)  :: uold2_wet3
    REAL(r_std),DIMENSION(npts,nvert),INTENT(in) :: uo_wet4
    REAL(r_std),DIMENSION(npts,nvert),INTENT(in)  :: uold2_wet4

    REAL(r_std),DIMENSION(npts),INTENT(in) :: tsurf_year
    !pss:-
!gmjc
  REAL(r_std),DIMENSION(npts,nvm),INTENT(in)    :: sla_calc
  REAL(r_std),DIMENSION(npts,nvm),INTENT(in)    :: wshtotsum
  REAL(r_std), DIMENSION(npts,nvm), INTENT(in)  ::  sr_ugb
!  REAL(r_std), DIMENSION(npts,nvm), INTENT(in)  ::  compt_ugb
  REAL(r_std),DIMENSION(npts,nvm),INTENT(in)    :: nb_ani
  REAL(r_std), DIMENSION(npts,nvm), INTENT(in)  ::  grazed_frac
  REAL(r_std), DIMENSION(npts,nvm), INTENT(in)  ::  import_yield
  REAL(r_std),DIMENSION(npts),INTENT(in)        :: t2m_14
  REAL(r_std),DIMENSION(npts,nlitt,nvm),INTENT(in)    :: litter_not_avail
  REAL(r_std), DIMENSION(npts,nvm), INTENT(in)  ::  nb_grazingdays
  REAL(r_std),DIMENSION(npts),INTENT(in)        :: after_snow
  REAL(r_std),DIMENSION(npts),INTENT(in)        :: after_wet
  REAL(r_std),DIMENSION(npts),INTENT(in)        :: wet1day
  REAL(r_std),DIMENSION(npts),INTENT(in)        :: wet2day
!end gmjc

  !spitfire
  ! Nesterov index accumulated
  REAL(r_std), DIMENSION(npts), INTENT(in)                    :: ni_acc
  ! fuel classes (1, 10, 100, 1000 hours)
  REAL(r_std), DIMENSION(npts,nvm,nlitt,nelements), INTENT(in)               :: fuel_1hr
  REAL(r_std), DIMENSION(npts,nvm,nlitt,nelements), INTENT(in)               :: fuel_10hr
  REAL(r_std), DIMENSION(npts,nvm,nlitt,nelements), INTENT(in)               :: fuel_100hr
  REAL(r_std), DIMENSION(npts,nvm,nlitt,nelements), INTENT(in)               :: fuel_1000hr
  !endspit

    !-
    ! 0.2 local
    !-
    ! date, real
    REAL(r_std) :: date_real
    ! PFT exists (equivalent to fpc_max > 0 for natural PFTs), real
    REAL(r_std),DIMENSION(npts,nvm) :: PFTpresent_real
    ! is the plant senescent ?
    ! (only for deciduous trees - carbohydrate reserve), real
    REAL(r_std),DIMENSION(npts,nvm) :: senescence_real

    REAL(r_std),DIMENSION(npts,nvm) :: begin_leaves_real

    ! in order for this PFT to be introduced,
    ! does it have to be present in an adjacent grid box? - real
    REAL(r_std),DIMENSION(npts,nvm) :: need_adjacent_real
    ! To store variables names for I/O
    CHARACTER(LEN=80) :: var_name
    ! string suffix indicating an index
    CHARACTER(LEN=10) :: part_str
    ! string suffix indicating biomass type crops
    CHARACTER(LEN=10) :: box_str
    ! string suffix indicating litter type
    CHARACTER(LEN=3),DIMENSION(nlitt) :: litter_str
    ! string suffix indicating level
    CHARACTER(LEN=2),DIMENSION(nlevs) :: level_str
    CHARACTER(LEN=2) :: layer_str    
    ! temporary storage
    REAL(r_std),DIMENSION(1) :: xtmp
    REAL(r_std), DIMENSION(1) :: vartmp  !! temporary variable because restget/restput needs a variable with DIMESION(:)
    ! index
    INTEGER(i_std) :: j,k,l,m,h 
    CHARACTER(LEN=1),DIMENSION(nelements) :: element_str  !! string suffix indicating element
    REAL(r_std), DIMENSION(1) :: temp_global_years
    CHARACTER(LEN=6),DIMENSION(npool) :: pools_str
    REAL(r_std), DIMENSION(npts) :: ok_equilibrium_real    

    ! land cover change variables 
    ! products remaining in the 10/100 year-turnover pool after the annual release for each compartment
    ! (10 or 100 + 1 : input from year of land cover change)
    REAL(r_std),DIMENSION(npts,0:10,nwp),INTENT(in)                           :: prod10
    REAL(r_std),DIMENSION(npts,0:100,nwp),INTENT(in)                          :: prod100
    ! annual release from the 10/100 year-turnover pool compartments
    REAL(r_std),DIMENSION(npts,10,nwp),INTENT(in)                           :: flux10
    REAL(r_std),DIMENSION(npts,100,nwp),INTENT(in)                          :: flux100
    REAL(r_std), DIMENSION(npts,nwp), INTENT(in)                            :: convflux
    REAL(r_std), DIMENSION(npts,nwp), INTENT(in)                            :: cflux_prod10
    REAL(r_std), DIMENSION(npts,nwp), INTENT(in)                            :: cflux_prod100
    REAL(r_std),DIMENSION(npts,nvm,nparts,nelements),INTENT(in)         :: bm_to_litter
    REAL(r_std),DIMENSION(npts),INTENT(in)                              :: carb_mass_total

    !---------------------------------------------------------------------
    IF (printlev >= 3) WRITE(numout,*) 'Entering writerestart'
    !-
    ! 1 string definitions
    !-
    DO l=1,nlitt
       IF     (l == imetabolic) THEN
          litter_str(l) = 'met'
       ELSEIF (l == istructural) THEN
          litter_str(l) = 'str'
       ELSE
          CALL ipslerr_p(3,'stomate_io writerestart','Define litter_str','','')
       ENDIF
    ENDDO
    !-
    DO l=1,nlevs
       IF     (l == iabove) THEN
          level_str(l) = 'ab'
       ELSEIF (l == ibelow) THEN
          level_str(l) = 'be'
       ELSE
          CALL ipslerr_p(3,'stomate_io writerestart','Define level_str','','')
       ENDIF
    ENDDO
    !-
    DO l=1,nelements
       IF     (l == icarbon) THEN
          element_str(l) = ''
!!$       ELSEIF (l == initrogen) THEN
!!$          element_str(l) = '_n'
       ELSE
          CALL ipslerr_p(3,'stomate_io writerestart','Define element_str','','')
       ENDIF
    ENDDO
    !-
    pools_str(1:npool) =(/'str_ab','str_be','met_ab','met_be','actif ','slow  ','passif'/)
    !-
    CALL ioconf_setatt_p ('UNITS','-')
    CALL ioconf_setatt_p ('LONG_NAME',' ')
    !-
    ! 2 run control
    !-
    ! 2.2 time step of STOMATE in days
    !-
!    IF (is_root_prc) THEN
!       var_name = 'dt_days'
!       xtmp(1) = dt_days
!       CALL restput (rest_id_stomate, var_name, 1, 1, 1, itime, xtmp)
!    ENDIF
    CALL restput_p (rest_id_stomate, 'dt_days', itime, dt_days)
    !-
    ! 2.3 date
    !-
!    IF (is_root_prc) THEN
!       var_name = 'date'
!       date_real = REAL(date,r_std)
!       xtmp(1) = date_real
!       CALL restput (rest_id_stomate, var_name, 1, 1, 1, itime, xtmp)
!    ENDIF
    CALL restput_p (rest_id_stomate, 'date', itime, date)
    !-
    ! 3 daily meteorological variables
    !-
    var_name = 'moiavail_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                moiavail_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'gdd_init_date'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    2, 1, itime, &
         &              gdd_init_date, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'litterhum_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                litterhum_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 't2m_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                t2m_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 't2m_min_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                t2m_min_daily, 'scatter', nbp_glo, index_g)
    !spitfire
    !-
    var_name = 't2m_max_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                t2m_max_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'wspeed_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                wspeed_daily, 'scatter', nbp_glo, index_g)
    !endspit
    !-
    var_name = 'tsurf_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                tsurf_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'tsoil_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nbdl, 1, itime, &
         &                tsoil_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'soilhum_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nbdl, 1, itime, &
         &                soilhum_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'precip_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                precip_daily, 'scatter', nbp_glo, index_g)

    ! Wetland CH4 methane
    !pss:+
    var_name = 'uo_0'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvert, 1, itime, &
         &              uo_0, 'scatter', nbp_glo, index_g)
    
    var_name = 'uold2_0'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvert, 1, itime, &
         &              uold2_0, 'scatter', nbp_glo, index_g)
    
    var_name = 'uo_wet1'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvert, 1, itime, &
         &              uo_wet1, 'scatter', nbp_glo, index_g)
    
    var_name = 'uold2_wet1'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvert, 1, itime, &
         &              uold2_wet1, 'scatter', nbp_glo, index_g)
    
    var_name = 'uo_wet2'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvert, 1, itime, &
         &              uo_wet2, 'scatter', nbp_glo, index_g)
 
    var_name = 'uold2_wet2'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvert, 1, itime, &
         &              uold2_wet2, 'scatter', nbp_glo, index_g)

    var_name = 'uo_wet3'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvert, 1, itime, &
       &              uo_wet3, 'scatter', nbp_glo, index_g)
    
    var_name = 'uold2_wet3'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvert, 1, itime, &
         &              uold2_wet3, 'scatter', nbp_glo, index_g)
    
    var_name = 'uo_wet4'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvert, 1, itime, &
         &              uo_wet4, 'scatter', nbp_glo, index_g)
    
    var_name = 'uold2_wet4'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvert, 1, itime, &
         &              uold2_wet4, 'scatter', nbp_glo, index_g)
 
    var_name = 'tsurf_year'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &              tsurf_year, 'scatter', nbp_glo, index_g)
   
    !pss:-


    !-
    ! 4 productivities
    !-
    var_name = 'gpp_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                gpp_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'npp_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                npp_daily, 'scatter', nbp_glo, index_g)
    !-
    CALL restput_p (rest_id_stomate, 'turnover_daily', nbp_glo, nvm, nparts, nelements, itime, &
       &                   turnover_daily, 'scatter', nbp_glo, index_g)
    !-
    ! 5 monthly meteorological variables
    !-
    var_name = 'moiavail_month'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                moiavail_month, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'moiavail_week'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                moiavail_week, 'scatter', nbp_glo, index_g)
    !-
    var_name = 't2m_longterm'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                t2m_longterm, 'scatter', nbp_glo, index_g)
    
!    IF (is_root_prc) THEN
!       var_name='tau_longterm'
!       vartmp(1)=tau_longterm
!       CALL restput (rest_id_stomate, var_name, 1, 1, 1, itime, vartmp)
!    ENDIF
    CALL restput_p (rest_id_stomate, 'tau_longterm', itime, tau_longterm)
       

    var_name = 't2m_month'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
                         t2m_month, 'scatter', nbp_glo, index_g)
    

    CALL restput_p (rest_id_stomate, 'Tseason', nbp_glo,    1, 1, itime, &
         Tseason, 'scatter', nbp_glo, index_g)
    
    CALL restput_p (rest_id_stomate, 'Tseason_length', nbp_glo,    1, 1, itime, &
         Tseason_length, 'scatter', nbp_glo, index_g)
    
    CALL restput_p (rest_id_stomate, 'Tseason_tmp', nbp_glo,    1, 1, itime, &
         Tseason_tmp, 'scatter', nbp_glo, index_g)
    
    CALL restput_p (rest_id_stomate, 'Tmin_spring_time', nbp_glo, nvm, 1, itime, &
         Tmin_spring_time, 'scatter', nbp_glo, index_g)
    
    var_name = 'Tseason'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                Tseason, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'Tseason_length'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                Tseason_length, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'Tseason_tmp'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                Tseason_tmp, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'Tmin_spring_time'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                Tmin_spring_time, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'onset_date'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                onset_date, 'scatter', nbp_glo, index_g)
    !-

    var_name = 't2m_week'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                t2m_week, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'tsoil_month'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nbdl, 1, itime, &
         &                tsoil_month, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'soilhum_month'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nbdl, 1, itime, &
         &                soilhum_month, 'scatter', nbp_glo, index_g)
    !-
    ! 6 fire probability
    !-
    var_name = 'fireindex'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                fireindex, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'firelitter'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                firelitter, 'scatter', nbp_glo, index_g)
    !-
    ! 7 maximum and minimum moisture availabilities for tropic phenology
    !-
    var_name = 'maxmoistr_last'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxmoiavail_lastyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'maxmoistr_this'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxmoiavail_thisyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'minmoistr_last'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                minmoiavail_lastyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'minmoistr_this'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                minmoiavail_thisyear, 'scatter', nbp_glo, index_g)
    !-
    ! 8 maximum "weekly" GPP
    !-
    var_name = 'maxgppweek_lastyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxgppweek_lastyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'maxgppweek_thisyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxgppweek_thisyear, 'scatter', nbp_glo, index_g)
    !-
    ! 9 annual GDD0
    !-
    var_name = 'gdd0_thisyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &                gdd0_thisyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'gdd0_lastyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &                gdd0_lastyear, 'scatter', nbp_glo, index_g)
    !-
    ! 10 annual precipitation
    !-
    var_name = 'precip_thisyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &                precip_thisyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'precip_lastyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &                precip_lastyear, 'scatter', nbp_glo, index_g)
    !-
    ! 11 derived "biometeorological" variables
    !-
    var_name = 'gdd_m5_dormance'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                gdd_m5_dormance, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'gdd_from_growthinit'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              gdd_from_growthinit, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'gdd_midwinter'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                gdd_midwinter, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'ncd_dormance'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                ncd_dormance, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'ngd_minus5'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                ngd_minus5, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'time_hum_min'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                time_hum_min, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'hum_min_dormance'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                hum_min_dormance, 'scatter', nbp_glo, index_g)
    !-
    ! 12 Plant status
    !-
    var_name = 'PFTpresent'
    WHERE ( PFTpresent(:,:) )
       PFTpresent_real = un
    ELSEWHERE
       PFTpresent_real = zero
    ENDWHERE
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                PFTpresent_real, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'ind'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                ind, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'turnover_time'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                turnover_time, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'adapted'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                adapted, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'regenerate'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                regenerate, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'npp_longterm'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                npp_longterm, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'lm_lastyearmax'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                lm_lastyearmax, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'lm_thisyearmax'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                lm_thisyearmax, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'maxfpc_lastyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxfpc_lastyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'maxfpc_thisyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxfpc_thisyear, 'scatter', nbp_glo, index_g)
    !-
    CALL restput_p (rest_id_stomate, 'turnover_longterm', nbp_glo, nvm, nparts, nelements, itime, &
         &                   turnover_longterm, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'gpp_week'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                gpp_week, 'scatter', nbp_glo, index_g)
    !-
    CALL restput_p (rest_id_stomate, 'biomass', nbp_glo, nvm, nparts, nelements, itime, &
          &                   biomass, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'maint_resp'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nparts, itime, &
         &                   resp_maint_part, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'leaf_age'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nleafages, itime, &
         &                  leaf_age, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'leaf_frac'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nleafages, itime, &
         &                   leaf_frac, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'senescence'
    WHERE ( senescence(:,:) )
       senescence_real = un
    ELSEWHERE
       senescence_real = zero
    ENDWHERE
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                senescence_real, 'scatter', nbp_glo, index_g)
 
    ! Transform the logical variable begin_leaves to real before writing to restart file
    var_name = 'begin_leaves'
    WHERE ( begin_leaves(:,:) )
       begin_leaves_real = un
    ELSEWHERE
       begin_leaves_real = zero
    ENDWHERE
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                begin_leaves_real, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'when_growthinit'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                when_growthinit, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'age'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                age, 'scatter', nbp_glo, index_g)
    !-
    ! 13 CO2
    !-
    var_name = 'resp_hetero'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                resp_hetero, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'tot_soil_resp'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                tot_soil_resp, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'resp_maint'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                resp_maint, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'resp_growth'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                resp_growth, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'co2_fire'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,  nvm, 1, itime, &
         &                co2_fire, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'co2_to_bm_dgvm'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                co2_to_bm_dgvm, 'scatter', nbp_glo, index_g)
    !-
    ! 14 vegetation distribution after last light competition
    !-
    var_name = 'veget_lastlight'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                veget_lastlight, 'scatter', nbp_glo, index_g)
    !-
    ! 15 establishment criteria
    !-
    var_name = 'everywhere'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                everywhere, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'need_adjacent'
    WHERE (need_adjacent(:,:))
       need_adjacent_real = un
    ELSEWHERE
       need_adjacent_real = zero
    ENDWHERE
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                need_adjacent_real, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'RIP_time'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                RIP_time, 'scatter', nbp_glo, index_g)
    !-
    ! 17 litter
    !-
    var_name = 'litterpart'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,  nvm, nlitt, itime, &
         &                   litterpart, 'scatter', nbp_glo, index_g)
    !-
    CALL restput_p (rest_id_stomate, 'litter_above', nbp_glo, nlitt, nvm, nelements, itime, &
         &                     litter_above, 'scatter', nbp_glo, index_g)
    !-
    CALL restput_p (rest_id_stomate, 'litter_below', nbp_glo, nlitt, nvm, ndeep, nelements, itime, &
         &                     litter_below, 'scatter', nbp_glo, index_g)
    !-    
    var_name = 'dead_leaves'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,  nvm, nlitt, itime, &
         &                   dead_leaves, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'carbon'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, ncarb, nvm, itime, &
         &                   carbon, 'scatter', nbp_glo, index_g)
!!SIMON ADDED ORCHIDOC
!    var_name = 'carbon_32l'
!    CALL restput_p (rest_id_stomate, var_name, nbp_glo, ncarb, nvm, ndeep, itime, &
!         &                   carbon_32l, 'scatter', nbp_glo, index_g)
 
   var_name = 'carbon_32l_a'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, ndeep, itime, &
         &               carbon_32l(:,iactive,:,:), 'scatter', nbp_glo, index_g)
    var_name = 'carbon_32l_s'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,  nvm, ndeep, itime, &
         &               carbon_32l(:,islow,:,:), 'scatter', nbp_glo, index_g)
    var_name = 'carbon_32l_p'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,  nvm, ndeep, itime, &
         &               carbon_32l(:,ipassive,:,:), 'scatter', nbp_glo, index_g) 
    !
    var_name = 'freedoc'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm , ndeep, npool, nelements, itime, &
         & DOC(:,:,:,ifree,:,:), 'scatter', nbp_glo, index_g)
    !
    var_name = 'adsdoc'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm , ndeep, npool, nelements, itime, &
         & DOC(:,:,:,iadsorbed,:,:), 'scatter', nbp_glo, index_g)
    !-
    var_name = 'lignin_struc_above'
    CALL restput_p &
        &      (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
        &       lignin_struc_above(:,:), 'scatter', nbp_glo, index_g)

    var_name = 'lig_struc_be'
    CALL restput_p &
        &      (rest_id_stomate, var_name, nbp_glo, nvm, ndeep, itime, &
        &       lignin_struc_below, 'scatter', nbp_glo, index_g)

    ! TF-DOC
    var_name = 'interception_storage'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,   nvm, 1, itime, &
        &                interception_storage(:,:,icarbon), 'scatter', nbp_glo, index_g)
   
!
!        var_name = 'lignin_struc'
!        CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nlevs, itime, &
!             &       lignin_struc, 'scatter', nbp_glo, index_g)
!!!SIMON ORCHIDOC END
    !spitfire 
       var_name = 'ni_acc'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1,1, itime, &
    &                ni_acc(:), 'scatter', nbp_glo, index_g)

    !-
    CALL restput_p (rest_id_stomate, 'fuel_1hr', nbp_glo, nvm  , nlitt, nelements, itime, &
            &       fuel_1hr, 'scatter', nbp_glo, index_g)

    CALL restput_p (rest_id_stomate, 'fuel_10hr', nbp_glo, nvm  , nlitt, nelements, itime, &
            &       fuel_10hr, 'scatter', nbp_glo, index_g)

    CALL restput_p (rest_id_stomate, 'fuel_100hr', nbp_glo, nvm  , nlitt, nelements, itime, &
            &       fuel_100hr, 'scatter', nbp_glo, index_g)

    CALL restput_p (rest_id_stomate, 'fuel_1000hr', nbp_glo, nvm  , nlitt, nelements, itime, &
            &       fuel_1000hr, 'scatter', nbp_glo, index_g)

    !endspit  
    !-
    !-
    ! 18 land cover change
    !-
    var_name = 'prod10'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 11, nwp, itime, &
         &                prod10, 'scatter', nbp_glo, index_g)
    var_name = 'prod100'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 101, nwp, itime, &
         &                prod100, 'scatter', nbp_glo, index_g)
    var_name = 'flux10'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 10, nwp, itime, &
         &                flux10, 'scatter', nbp_glo, index_g)
    var_name = 'flux100'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 100, nwp, itime, &
         &                flux100, 'scatter', nbp_glo, index_g)

 !   var_name = 'convflux'
 !   CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, nwp, itime, &
 !        &              convflux, 'scatter', nbp_glo, index_g)
 !   var_name = 'cflux_prod10'
 !   CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, nwp, itime, &
 !        &              cflux_prod10, 'scatter', nbp_glo, index_g)
 !   var_name = 'cflux_prod100'
 !   CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, nwp, itime, &
 !        &              cflux_prod100, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id_stomate, 'bm_to_litter', nbp_glo, nvm, nparts, nelements, itime, &
        &                bm_to_litter, 'scatter', nbp_glo, index_g)

    var_name = 'carb_mass_total'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &              carb_mass_total, 'scatter', nbp_glo, index_g)

    ! 19 Permafrost carbon related
    var_name = 'deepC_a'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, ndeep, nvm, itime, &
         &               deepC_a, 'scatter', nbp_glo, index_g)
    var_name = 'deepC_s'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, ndeep, nvm, itime, &
         &               deepC_s, 'scatter', nbp_glo, index_g)
    var_name = 'deepC_p'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, ndeep, nvm, itime, &
         &               deepC_p, 'scatter', nbp_glo, index_g)
    var_name = 'O2_soil'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, ndeep, nvm, itime, &
         &               O2_soil, 'scatter', nbp_glo, index_g)
    var_name = 'CH4_soil'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, ndeep, nvm, itime, &
         &               CH4_soil, 'scatter', nbp_glo, index_g)
    var_name = 'O2_snow'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nsnow, nvm, itime, &
         &               O2_snow, 'scatter', nbp_glo, index_g)
    var_name = 'CH4_snow'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nsnow, nvm, itime, &
         &               CH4_snow, 'scatter', nbp_glo, index_g)

    var_name = 'thawed_humidity'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
   &               thawed_humidity, 'scatter', nbp_glo, index_g)

    var_name = 'depth_organic_soil'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
   &               depth_organic_soil, 'scatter', nbp_glo, index_g)

    var_name = 'altmax'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
   &               altmax, 'scatter', nbp_glo, index_g)
   !Isa dbg : fixed_cryoturbation_depth -> fixed_cryoturb_depth
    var_name = 'fixed_cryoturb_depth'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
   &               fixed_cryoturbation_depth, 'scatter', nbp_glo, index_g)

!gmjc
!-
!  var_name = 'sla_calc'
!  CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
! &              sla_calc, 'scatter', nbp_glo, index_g)
!-
  var_name = 'wshtotsum'
  CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
 &              wshtotsum, 'scatter', nbp_glo, index_g)
!-
  var_name = 'sr_ugb'
  CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
 &              sr_ugb, 'scatter', nbp_glo, index_g)
!-
  var_name = 'sla_calc'
  CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
 &              sla_calc, 'scatter', nbp_glo, index_g)
!-
  var_name = 'nb_ani'
  CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
 &              nb_ani, 'scatter', nbp_glo, index_g)
!-
  var_name = 'grazed_frac'
  CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
 &              grazed_frac, 'scatter', nbp_glo, index_g)
!-
  var_name = 'import_yield'
  CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
 &              import_yield, 'scatter', nbp_glo, index_g)
!-
  var_name = 't2m_14'
  CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
 &              t2m_14, 'scatter', nbp_glo, index_g)

  CALL restput_p (rest_id_stomate, 'litter_not_avail', nbp_glo,  nlitt, nvm, itime, &
 &                   litter_not_avail, 'scatter', nbp_glo, index_g)
!-
  var_name = 'nb_grazingdays'
  CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
 &              nb_grazingdays, 'scatter', nbp_glo, index_g)
!-
!-
  var_name = 'after_snow'
  CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
 &              after_snow, 'scatter', nbp_glo, index_g)
!-
  var_name = 'after_wet'
  CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
 &              after_wet, 'scatter', nbp_glo, index_g)
!-
  var_name = 'wet1day'
  CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
 &              wet1day, 'scatter', nbp_glo, index_g)
!-
  var_name = 'wet2day'
  CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
 &              wet2day, 'scatter', nbp_glo, index_g)
!end gmjc
 
    !-
    ! 19. Spinup
    !-
!    IF (spinup_analytic) THEN

 !      IF (is_root_prc) THEN
 !         temp_global_years(1) = REAL(global_years)
!          var_name='Global_years'
 !         call restput (rest_id_stomate, var_name, 1, 1, 1, itime, temp_global_years)
 !      ENDIF
       CALL restput_p (rest_id_stomate, 'Global_years', itime, global_years)
       
       var_name = 'nbp_sum'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
            &              nbp_accu, 'scatter', nbp_glo, index_g)

       var_name = 'nbp_flux'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
            &              nbp_flux, 'scatter', nbp_glo, index_g)

       var_name = 'ok_equilibrium'
       WHERE(ok_equilibrium(:))
          ok_equilibrium_real = un
       ELSEWHERE
          ok_equilibrium_real = zero
       ENDWHERE
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
            &               ok_equilibrium_real, 'scatter', nbp_glo, index_g)
       
       CALL restput_p (rest_id_stomate, 'MatrixV', nbp_glo, nvm, npool, npool, itime, &
            &                MatrixV, 'scatter', nbp_glo, index_g)
          
       var_name = 'Vector_U'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, npool, itime, &
            &                VectorU, 'scatter', nbp_glo, index_g)
          
       var_name = 'previous_stock'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, npool, itime, &
            &                previous_stock, 'scatter', nbp_glo, index_g)
          
       var_name = 'current_stock'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, npool, itime, &
            &                current_stock, 'scatter', nbp_glo, index_g)

!    ENDIF !(spinup_analytic)
    !-


       var_name = 'assim_param'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, npco2, itime, &
            &                assim_param, 'scatter', nbp_glo, index_g)
       

    IF (printlev >= 4) WRITE(numout,*) 'Leaving writerestart'
    !--------------------------
  END SUBROUTINE writerestart
  !-
  !===
  !-
  SUBROUTINE get_reftemp_clear
    !---------------------------------------------------------------------
    firstcall_io=.TRUE.
    IF (ALLOCATED (trefe)) DEALLOCATE( trefe )
    !-------------------------------
  END SUBROUTINE get_reftemp_clear
  !-
  !===
  !-
  SUBROUTINE get_reftemp (npts, lalo, resolution, filename, tref_out)
    !---------------------------------------------------------------------
    !- read the long-term reference temperature from a boundary condition
    !- file. If the vegetation is dynamic, this field is used to
    !- initialize correctly the (prognostic) long-term reference
    !- temperature (in the case it is not found in the restart file).
    !- If the vegetation is static, the field read here is a real boundary
    !- condition that is not modified by the model.
    !---------------------------------------------------------------------
    !-
    ! 0 declarations
    !-
    ! 0.1 input
    !-
    ! Domain size
    INTEGER(i_std),INTENT(in) :: npts
    ! Geogr. coordinates (latitude,longitude) (degrees)
    REAL(r_std),DIMENSION (npts,2),INTENT(in) :: lalo
    ! size in x an y of the grid (m)
    REAL(r_std),DIMENSION (npts,2),INTENT(in) :: resolution
    CHARACTER(LEN=80), INTENT(IN) :: filename
    !-
    ! 0.2 output
    !-
    ! reference temperature (K)
    REAL(r_std), DIMENSION(npts),INTENT(out) :: tref_out
    !-
    ! 0.3 local
    !-
    INTEGER(i_std),PARAMETER :: nbvmax=200
    INTEGER(i_std) :: &
         &  iml, jml, lml, tml, fid, ib, ip, jp, fopt, ilf, lastjp
    REAL(r_std) :: lev(1), date, dt, coslat
    INTEGER(i_std)                                 :: itau(1)
    REAL(r_std),ALLOCATABLE,DIMENSION(:,:) :: &
         &  lat_rel, lon_rel, lat_ful, lon_ful, tref_file
    REAL(r_std),ALLOCATABLE,DIMENSION(:,:) :: &
         &  loup_rel, lolow_rel, laup_rel, lalow_rel
    REAL(r_std) :: lon_up, lon_low, lat_up, lat_low
    REAL(r_std) :: ax, ay, sgn
    REAL(r_std),DIMENSION(nbvmax) :: area
    REAL(r_std),DIMENSION(nbvmax) :: tt
    REAL(r_std) :: resx, resy
    LOGICAL :: do_again
    !---------------------------------------------------------------------
    !-
    ! 1 If this is the first call, calculate the reference temperature
    !   and keep it in memory
    !-
    IF (firstcall_io) THEN
       !---
       !-- 1.1 only do this once
       !---
       firstcall_io = .FALSE.
       !---
       !-- 1.2 allocate the field
       !---
       ALLOCATE( trefe(npts) )
       !---
       !-- 1.3 read and interpolate the temperature file
       !---
       !-- Needs to be a configurable variable
       !---
       !---
       IF (is_root_prc) CALL flininfo(filename,iml, jml, lml, tml, fid)
       CALL bcast(iml)
       CALL bcast(jml)
       CALL bcast(lml)
       CALL bcast(tml)
       !---
       ALLOCATE (lat_rel(iml,jml))
       ALLOCATE (lon_rel(iml,jml))
       ALLOCATE (laup_rel(iml,jml))
       ALLOCATE (loup_rel(iml,jml))
       ALLOCATE (lalow_rel(iml,jml))
       ALLOCATE (lolow_rel(iml,jml))
       ALLOCATE (lat_ful(iml+2,jml+2))
       ALLOCATE (lon_ful(iml+2,jml+2))
       ALLOCATE (tref_file(iml,jml))
       !---
       IF (is_root_prc) CALL flinopen (filename, .FALSE., iml, jml, lml, &
            &                                   lon_rel, lat_rel, lev, tml, itau, date, dt, fid)
       CALL bcast(lon_rel)
       CALL bcast(lat_rel)
       CALL bcast(itau)
       CALL bcast(date)
       CALL bcast(dt)

       !---
       IF (is_root_prc) CALL flinget (fid, 'temperature', iml, jml, lml, tml, &
            &                                  1, 1, tref_file)
       CALL bcast(tref_file)
       !---
       IF (is_root_prc) CALL flinclo (fid)
       !---
       !-- Duplicate the border assuming we have a global grid
       !-- going from west to east
       !---
       lon_ful(2:iml+1,2:jml+1) = lon_rel(1:iml,1:jml)
       lat_ful(2:iml+1,2:jml+1) = lat_rel(1:iml,1:jml)
       !---
       IF ( lon_rel(iml,1) < lon_ful(2,2)) THEN
          lon_ful(1,2:jml+1) = lon_rel(iml,1:jml)
          lat_ful(1,2:jml+1) = lat_rel(iml,1:jml)
       ELSE
          lon_ful(1,2:jml+1) = lon_rel(iml,1:jml)-360
          lat_ful(1,2:jml+1) = lat_rel(iml,1:jml)
       ENDIF
       !---
       IF ( lon_rel(1,1) > lon_ful(iml+1,2)) THEN
          lon_ful(iml+2,2:jml+1) = lon_rel(1,1:jml)
          lat_ful(iml+2,2:jml+1) = lat_rel(1,1:jml)
       ELSE
          lon_ful(iml+2,2:jml+1) = lon_rel(1,1:jml)+360
          lat_ful(iml+2,2:jml+1) = lat_rel(1,1:jml)
       ENDIF
       !---
       sgn = lat_rel(1,1)/ABS(lat_rel(1,1))
       lat_ful(2:iml+1,1) = sgn*180 - lat_rel(1:iml,1)
       sgn = lat_rel(1,jml)/ABS(lat_rel(1,jml))
       lat_ful(2:iml+1,jml+2) = sgn*180 - lat_rel(1:iml,jml)
       lat_ful(1,1) = lat_ful(iml+1,1)
       lat_ful(iml+2,1) = lat_ful(2,1)
       lat_ful(1,jml+2) = lat_ful(iml+1,jml+2)
       lat_ful(iml+2,jml+2) = lat_ful(2,jml+2)
       !---
       !-- Add the longitude lines to the top and bottom
       !---
       lon_ful(:,1) = lon_ful(:,2)
       lon_ful(:,jml+2) = lon_ful(:,jml+1)
       !---
       !-- Get the upper and lower limits of each grid box
       !---
       DO ip=1,iml
          DO jp=1,jml
             loup_rel(ip,jp) = &
                  &        MAX(0.5*(lon_ful(ip,jp+1)+lon_ful(ip+1,jp+1)), &
                  &            0.5*(lon_ful(ip+1,jp+1)+lon_ful(ip+2,jp+1)))
             lolow_rel(ip,jp) = &
                  &        MIN(0.5*(lon_ful(ip,jp+1)+lon_ful(ip+1,jp+1)), &
                  &            0.5*(lon_ful(ip+1,jp+1)+lon_ful(ip+2,jp+1)))
             laup_rel(ip,jp) = &
                  &        MAX(0.5*(lat_ful(ip+1,jp)+lat_ful(ip+1,jp+1)), &
                  &            0.5*(lat_ful(ip+1,jp+1)+lat_ful(ip+1,jp+2)))
             lalow_rel(ip,jp) = &
                  &        MIN(0.5*(lat_ful(ip+1,jp)+lat_ful(ip+1,jp+1)), &
                  &            0.5*(lat_ful(ip+1,jp+1)+lat_ful(ip+1,jp+2)))
          ENDDO
       ENDDO
       !---
       !-- Now we take each grid point and find out which values
       !-- from the forcing we need to average
       !---
       DO ib=1,npts
          !-----
          resx = resolution(ib,1)
          resy = resolution(ib,2)
          !-----
          do_again = .TRUE.
          !-----
          DO WHILE (do_again)
             !-----
             do_again = .FALSE.
             !-------
             !------ We find the 4 limits of the grid-box.
             !------ As we transform the resolution of the model into longitudes
             !------ and latitudes we do not have the problem of periodicity.
             !------ coslat is a help variable here !
             !-------
             coslat = MAX(COS(lalo(ib,1)*pi/180.),mincos)*pi/180.*R_Earth
             !-------
             lon_up  = lalo(ib,2)+resx/(2.0*coslat)
             lon_low = lalo(ib,2)-resx/(2.0*coslat)
             !-------
             coslat  = pi/180.*R_Earth
             !-------
             lat_up  = lalo(ib,1)+resy/(2.0*coslat)
             lat_low = lalo(ib,1)-resy/(2.0*coslat)
             !-------
             !------ Find the grid boxes from the data that go into
             !------ the model's boxes.
             !------ We still work as if we had a regular grid !
             !------ Well it needs to be localy regular so that
             !------ the longitude at the latitude of the last found point
             !------ is close to the one of the next point.
             !-------
             fopt = 0
             lastjp = 1
             DO ip=1,iml
                !---------
                !-------- Either the center of the data grid point is in the interval
                !-------- of the model grid or the East and West limits of the data
                !-------- grid point are on either sides of the border of the data grid
                !---------
                IF (      lon_rel(ip,lastjp) > lon_low &
                     &            .AND. lon_rel(ip,lastjp) < lon_up &
                     &             .OR. lolow_rel(ip,lastjp) < lon_low &
                     &            .AND. loup_rel(ip,lastjp) > lon_low &
                     &             .OR. lolow_rel(ip,lastjp) < lon_up &
                     &            .AND. loup_rel(ip,lastjp) > lon_up ) THEN
                   DO jp=1,jml
                      !-------------
                      !------------ Now that we have the longitude let us find the latitude
                      !-------------
                      IF (      lat_rel(ip,jp) > lat_low &
                           &                 .AND. lat_rel(ip,jp) < lat_up &
                           &                  .OR. lalow_rel(ip,jp) < lat_low &
                           &                 .AND. laup_rel(ip,jp) > lat_low &
                           &                  .OR. lalow_rel(ip,jp) < lat_up &
                           &                 .AND. laup_rel(ip,jp) > lat_up) THEN
                         lastjp = jp
                         !---------------
                         fopt = fopt + 1
                         IF ( fopt > nbvmax) THEN
                            WRITE(numout,*) &
                                 &                       'Please increase nbvmax in subroutine get_reftemp',ib
                            CALL ipslerr_p(3,'stomate_io','get_reftemp','Please increase nbvmax','')
                         ELSE
                            !-----------------
                            !---------------- Get the area of the fine grid in the model grid
                            !-----------------
                            coslat = MAX(COS(lat_rel(ip,jp)*pi/180.),mincos)
                            ax =  ( MIN(lon_up,loup_rel(ip,jp)) &
                                 &                       -MAX(lon_low,lolow_rel(ip,jp))) &
                                 &                     *pi/180.*R_Earth*coslat
                            ay =  ( MIN(lat_up,laup_rel(ip,jp)) &
                                 &                       -MAX(lat_low,lalow_rel(ip,jp))) &
                                 &                     *pi/180.*R_Earth
                            area(fopt) = ax*ay
                            tt(fopt) = tref_file(ip,jp)
                         ENDIF
                      ENDIF
                   ENDDO
                ENDIF
             ENDDO
             !-------
             !------ Check that we found some points
             !-------
             trefe(ib) = zero
             !-------
             IF (fopt == 0) THEN
                do_again = .TRUE.
                !-------
                !------ increase search radius
                !-------
                resx = resx*2.
                resy = resy*2.
                IF ( resx > 2.*pi*R_Earth .OR. resy > pi*R_Earth ) THEN
                   CALL ipslerr_p(3,'stomate_io','get_reftemp: found no point 1','','')
                ENDIF
             ELSE
                sgn = zero
                !-------
                !------ Compute the average surface air temperature
                !-------
                DO ilf=1,fopt
                   trefe(ib) = trefe(ib) + tt(ilf) * area(ilf)
                   sgn = sgn + area(ilf)
                ENDDO
                !-------
                !------ Normalize the surface
                !-------
                IF (sgn < min_sechiba) THEN
                   do_again = .TRUE.
                   !---------
                   !-------- increase search radius
                   !---------
                   resx = resx * 2.
                   resy = resy * 2.
                   IF ( resx > 2.*pi*R_Earth .OR. resy > pi*R_Earth ) THEN
                      CALL ipslerr_p(3,'stomate_io','get_reftemp: found no point 2','','')
                   ENDIF
                ELSE
                   trefe(ib) = trefe(ib) / sgn
                ENDIF
             ENDIF
          ENDDO
       ENDDO
       !-
       ! transform into Kelvin
       !-
       trefe(:) = trefe(:) + ZeroCelsius
       !-
       ! deallocate
       !-
       DEALLOCATE (lat_rel)
       DEALLOCATE (lon_rel)
       DEALLOCATE (laup_rel)
       DEALLOCATE (loup_rel)
       DEALLOCATE (lalow_rel)
       DEALLOCATE (lolow_rel)
       DEALLOCATE (lat_ful)
       DEALLOCATE (lon_ful)
       DEALLOCATE (tref_file)
    ENDIF
    !-
    ! 2 output the reference temperature
    !-
    tref_out(:) = trefe(:)
    !-------------------------
  END SUBROUTINE get_reftemp
  !-
  !===
  !-
END MODULE stomate_io

